#!/usr/env/python
# -*- coding: utf-8 -*-
"""
gridhelper.py (c) 2009 rene.milk@uni-muenster.de
Licence: WTFPLv2, see LICENSE.txt
"""

import math
import copy
from euclid import *
import colors

class DimensionIncompatibleException(Exception):
	pass

class ImpossibleException(Exception):
	pass

class ColorToBoundaryIdMapper:
	def __init__(self):
		self.known_colors = []
		self.eps = 5e-1

	def getID(self, color):
		assert isinstance( color, Vector3 )
		idx = -1
		for c in self.known_colors:
			if abs(c - color) < self.eps:
				idx = self.known_colors.index(c)
				return idx + 1
		self.known_colors.append(color)
		return self.known_colors.index(color) + 1

	def __repr__(self):
		return '%d Colors: '%(len(self.known_colors)) + ' '.join( map(lambda p: str(p), self.known_colors ) )

class BoundaryIdToColorMapper:
	def __init__(self,expected=11):
		self.bids = []
		self.expected = expected
		self.colormap = colors.getColourPaletteCheat(expected,[(0.0,0,0.0)])

	def getColor(self, bid):
		if not bid in self.bids:
			self.bids.append(bid)
		try:
			c = self.colormap[ self.bids.index(bid) ]
			return Vector3( c[0], c[1], c[2] )
		except Exception, e:
			print e
			print bid
			return Vector3()

class Simplex3:
	def __init__(self,a,b,c,pl,f_id,color=None,boundary_id=None):
		assert isinstance(pl,MeshVertexList)
		assert isinstance(a,int)
		assert isinstance(b,int)
		assert isinstance(c,int)
		assert a!=b and b !=c and b!=c, 'degenerated simplex'
		max_id = len(pl)
		assert a > -1 and a <= max_id, 'negative vertex id: %d / %d / %d'%(a,b,c)
		assert b > -1 and b <= max_id, 'negative vertex id: %d / %d / %d'%(a,b,c)
		assert c > -1 and c <= max_id, 'negative vertex id: %d / %d / %d'%(a,b,c)
		self.attribs = ( pl.attribs[a], pl.attribs[b], pl.attribs[c] )
		self.idx = ( a,b,c )
		self.edge_idx = ( (a,b), (b,c), (c,a) )
		self.id = f_id
		self.reset(pl)
		self.m = Vector3()
		if color != None:
			self.color = color
		else:
			self.color = self.attribs[0]/255.0
		self.boundary_id = boundary_id

	def reset(self,pl):
		self.v = []
		self.center = Vector3()
		
		for id in self.idx:
			try:
				self.v.append( pl[id] )
				self.center += pl[id]
			except IndexError, e:
				import traceback
				print traceback.format_exc()
				print self.idx, id
				raise e
		self.center /= 3.0
		if self.v[0] < self.v[1]  and self.v[1] < self.v[2]:
			self.n = ( - self.v[0] + self.v[1] ).cross( - self.v[0] + self.v[2] )
			#elif self.v[0] < self.v[1]  and self.v[1] > self.v[2]:
				#self.n = (self.v[0] - self.v[1] ).cross(   self.v[1] - self.v[2] ) * -1
		else:
			self.n = (self.v[0] - self.v[1] ).cross(   self.v[1] - self.v[2] )
		n_abs = abs(self.n)
		if n_abs != 0.0:
			self.n /= n_abs
		self.edges = ( self.v[1] - self.v[0], self.v[2] - self.v[1], self.v[0] - self.v[2] )
		self.area  = n_abs * 0.5

	def __repr__(self):
		return 'simplex (%d,%d,%d) -- (%s,%s,%s)'%(self.idx[0],self.idx[1],self.idx[2],self.v[0],self.v[1],self.v[2])

def vector( x,y,z=None ):
	if z:
		return Vector3( float(x),float(y),float(z) )
	else:
		return Vector3( float(x),float(y), 0.0 )

class MeshVertexList(object):
	def __init__(self, dim=3):
		self.__vertices = dict()
		self.attribs = dict()
		#alias -> real vertex id mapping
		self.aliases = dict()
		self.duplicates = []

	def addVertex(self,v,c):
		assert isinstance( c, Vector3 )
		assert isinstance( v, Vector3 )
		next_vertex_id = len(self.__vertices) + len(self.duplicates)

		if not v in self.__vertices:
			self.__vertices[next_vertex_id] = v
			self.aliases[next_vertex_id] = next_vertex_id
		else:
			self.duplicates.append(v)
			self.aliases[next_vertex_id] = self.__vertices.index( v )
		self.attribs[next_vertex_id] = c
		return next_vertex_id

	def __getitem__(self,idx):
		assert idx in self.aliases.keys()
		real_idx = self.aliases[idx]
		#assert len(self.__vertices) >  real_idx , 'v %d -- r %d| len %d'%(idx, real_idx,len(self.__vertices))
		return self.__vertices[real_idx]

	def __setitem__(self,idx,v):
		assert isinstance(idx,int)
		assert idx > -1
		assert isinstance(v, Vector3)
		idx = self.aliases[idx]
		self.__vertices[idx] = v

	def realIndex(self,idx):
		if '__getitem__' in dir(idx):
			return map( lambda i: self.aliases[i], idx)
		else:
			return self.aliases[idx]

	def __len__(self):
		return len(self.__vertices)

	def getVertices(s):
		return s.__vertices.values()[:]

def find_key(dic, val):
	"""return the key of dictionary dic given the value"""
	return [k for k, v in dic.iteritems() if v == val][0]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
widgets.py (c) 2009 rene.milk@uni-muenster.de
Licence: WTFPLv2, see LICENSE.txt
"""

import sys
import time
import math
import Image
from OpenGL.GL import (glBegin, glVertex3fv, glPushMatrix, glTranslatef,
	glPopMatrix, glEndList, glCallList, glColor4f, glNewList,
	glClear, glClearColor, glColorMaterial, glBlendFunc, glClearDepth,
	glDisable, glEnable, glHint, glLightfv, glLoadIdentity, glMatrixMode,
	glReadPixels, glRotate, glShadeModel, glViewport, glDepthFunc, glEnd,
	GL_AMBIENT, GL_AMBIENT_AND_DIFFUSE, GL_BLEND, GL_COLOR_BUFFER_BIT,
	GL_COLOR_MATERIAL, GL_CULL_FACE, GL_DEPTH_BUFFER_BIT, GL_DEPTH_TEST,
	GL_DIFFUSE, GL_FRONT_AND_BACK, GL_LESS, GL_LIGHT0, GL_LIGHTING,
	GL_MODELVIEW, GL_MULTISAMPLE, GL_NICEST, GL_NORMALIZE, GL_ONE,
	GL_ONE_MINUS_SRC_ALPHA, GL_PERSPECTIVE_CORRECTION_HINT, GL_POSITION,
	GL_PROJECTION, GL_RGBA, GL_SMOOTH, GL_SPECULAR, GL_SRC_ALPHA,
	GL_TRIANGLES, GL_UNSIGNED_BYTE)
from OpenGL.GLU import (gluNewQuadric, gluPerspective, gluSphere)
from PyQt4 import QtCore, QtGui, QtOpenGL
from dune.mesh import mesh 

class BidPanel(object):
	def __init__(self, parent, mesh):
		super(BidPanel, self).__init__()
		for bid, colors in mesh.bid_color_mapping.iteritems():
			print('BID %d: %s' % (bid,' '.join(set(
						[str((255*c.x,255*c.y,255*c.z)) for c in colors])) ))
		
class ControlPanel(QtGui.QWidget):
	def __init__(self, parent, bid_widget):
		super(ControlPanel, self).__init__(parent)
		box = QtGui.QVBoxLayout()

		assert parent
		self.viewer = parent

		self.setWindowTitle("Control Panel")
		self.resize(200, 320)
		groupBox = QtGui.QGroupBox("Smoothing parameters")
		grid = QtGui.QGridLayout()
		groupBox.setLayout( grid )
		box.addWidget( groupBox )
		lambdaLabel = QtGui.QLabel("Step Size (lambda)")
		grid.addWidget( lambdaLabel, 0, 0 )
		self.lambdaSpinBox = QtGui.QDoubleSpinBox()
		self.lambdaSpinBox.setRange(0.0, 20.0)
		self.lambdaSpinBox.setSingleStep(0.01)
		self.lambdaSpinBox.setValue(0.01)
		grid.addWidget( self.lambdaSpinBox, 0, 1 )
		iterationsLabel = QtGui.QLabel("# iterations")
		grid.addWidget( iterationsLabel, 1, 0 )
		self.iterationsSpinBox = QtGui.QSpinBox()
		self.iterationsSpinBox.setRange(1, 120.0)
		self.iterationsSpinBox.setSingleStep(1)
		self.iterationsSpinBox.setValue(1)
		grid.addWidget( self.iterationsSpinBox, 1, 1 )

		grid2 = QtGui.QGridLayout()
		groupBox2 = QtGui.QGroupBox("Smoothing algorithm")
		laplaceButton = QtGui.QPushButton("L&aplace")
		laplaceButton.setFocusPolicy(QtCore.Qt.NoFocus)
		laplaceButton.clicked.connect(self.viewer.smoothLaplace)
		grid2.addWidget( laplaceButton, 0, 0 )
		normalButton = QtGui.QPushButton("Mean &Normal")
		normalButton.setFocusPolicy(QtCore.Qt.NoFocus)
		normalButton.clicked.connect(self.viewer.smoothMeanNormal)
		grid2.addWidget( normalButton, 1,0 )
		random_noiseButton = QtGui.QPushButton("Random No&ise")
		random_noiseButton.setFocusPolicy(QtCore.Qt.NoFocus)
		random_noiseButton.clicked.connect(self.viewer.noise)
		grid2.addWidget( random_noiseButton, 2,0 )

		groupBox2.setLayout( grid2 )
		box.addWidget( groupBox2 )

		grid3 = QtGui.QGridLayout()
		groupBox3 = QtGui.QGroupBox("View")
		resetButton = QtGui.QPushButton("&Reset")
		resetButton.setFocusPolicy(QtCore.Qt.NoFocus)
		resetButton.clicked.connect(self.viewer.reset)
		grid3.addWidget( resetButton, 0,0 )
		self.draw_bounding_box = QtGui.QCheckBox("Show &bounding box", self)
		self.draw_bounding_box.stateChanged.connect(self.viewer.setOptions)
		grid3.addWidget( self.draw_bounding_box, 1,0 )
		self.draw_mesh = QtGui.QCheckBox("Show &mesh", self)
		self.draw_mesh.stateChanged.connect(self.viewer.setOptions)
		grid3.addWidget( self.draw_mesh, 2,0 )
		#self.point_num = QtGui.QSpinBox()
		#self.point_num.setRange(0, 8000.0)
		#self.point_num.setSingleStep(1)
		#self.point_num.setValue(1)
		#self.point_num.valueChanged.connect(self.viewer.setPointDraw)
		#grid3.addWidget( self.point_num, 2,0 )
		subbox1 = QtGui.QGroupBox("Restrict")
		subgrid1 = QtGui.QGridLayout()
		self.enable_restrict = QtGui.QCheckBox("Enable", self)
		self.enable_restrict.stateChanged.connect(self.viewer.enable_restrict)
		subgrid1.addWidget(  self.enable_restrict, 0, 0 )
		self.slider_min = QtGui.QSlider(QtCore.Qt.Horizontal)
		self.slider_max = QtGui.QSlider(QtCore.Qt.Horizontal)
		self.setMinMaxSlider()
		subgrid1.addWidget(  self.slider_min, 1, 0 )
		subgrid1.addWidget(  self.slider_max, 2, 0 )
		subbox1.setLayout( subgrid1 )
		grid3.addWidget( subbox1, 3,0 )

		groupBox3.setLayout( grid3 )
		box.addWidget( groupBox3 )

		grid4 = QtGui.QGridLayout()
		groupBox4 = QtGui.QGroupBox("Save/Load")
		loadButton = QtGui.QPushButton("&Load")
		loadButton.setFocusPolicy(QtCore.Qt.NoFocus)
		loadButton.clicked.connect(self.viewer.load)
		grid4.addWidget( loadButton, 0,0 )
		saveButton = QtGui.QPushButton("Sa&ve")
		saveButton.setFocusPolicy(QtCore.Qt.NoFocus)
		saveButton.clicked.connect(self.viewer.save)
		grid4.addWidget( saveButton, 1,0 )
		reloadButton = QtGui.QPushButton("R&eload")
		reloadButton.setFocusPolicy(QtCore.Qt.NoFocus)
		reloadButton.clicked.connect(self.viewer.reload)
		grid4.addWidget( reloadButton, 2,0 )
		screenieButton = QtGui.QPushButton("Screenshot")
		screenieButton.setFocusPolicy(QtCore.Qt.NoFocus)
		screenieButton.clicked.connect(self.viewer.saveScreenshot)
		grid4.addWidget( screenieButton, 3,0 )
		groupBox4.setLayout( grid4 )
		box.addWidget( groupBox4 )

		#box.addWidget(bid_widget)
		self.setLayout(box)

		#connect only now lest be called on manual set
		self.slider_max.valueChanged.connect(self.viewer.set_restrict)
		self.slider_min.valueChanged.connect(self.viewer.set_restrict)

	def setMinMaxSlider(self):
		mx = len(self.viewer.widget.mesh.faces)
		for sl in [ self.slider_min , self.slider_max ]:
			sl.setMinimum( 0 )
			sl.setMaximum( mx )
			sl.setFocusPolicy(QtCore.Qt.StrongFocus)
			sl.setTickInterval(mx/10)
			sl.setSingleStep(1)
		self.slider_min.setValue(0)
		self.slider_min.setTickPosition(QtGui.QSlider.TicksAbove)
		self.slider_max.setTickPosition(QtGui.QSlider.TicksBelow)
		self.slider_max.setValue(mx)


class MeshWidget(QtOpenGL.QGLWidget):
	GL_MULTISAMPLE = 0x809D

	def initializeGL(self):
		# We call this right after our OpenGL window is created.
		glClearColor(1.0, 1.0, 1.0, 1.0)
		glClearDepth(1.0)
		glDepthFunc(GL_LESS)
		glEnable(GL_DEPTH_TEST)
		glShadeModel(GL_SMOOTH)

		glEnable(GL_NORMALIZE)
		light_position = (0., 0., 1., 0.)
		white_light = (1., 1., 1., 0.501)
		d_light = (1., 1., 1., 0.01)
		spec = (1., 1., 1., 0.08)
		glLightfv(GL_LIGHT0, GL_POSITION, light_position)
		glLightfv(GL_LIGHT0, GL_AMBIENT,  white_light)
		#glLightfv(GL_LIGHT0, GL_DIFFUSE,  d_light)
		glLightfv(GL_LIGHT0, GL_SPECULAR, spec)

		glEnable(GL_LIGHTING)
		glEnable(GL_LIGHT0)

		glEnable(GL_BLEND)
		glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA)
		#glBlendFunc(GL_SRC_ALPHA,GL_ONE)
		glColorMaterial(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE)
		glEnable(GL_COLOR_MATERIAL)
		glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST)

		glMatrixMode(GL_PROJECTION)
		glLoadIdentity()

		gluPerspective(45.0, float(self.size().height())/
						float(self.size().width()), 0.1, 1000000.0)

		glMatrixMode(GL_MODELVIEW)
		glLoadIdentity()
		self.rotation = [0.0, 0.0]
		self.mesh.prepDraw()
		#glEnable(GL_CULL_FACE)

	def resizeGL(self, w, h):
		# Prevent A Divide By Zero If The Window Is Too Small
		h = max(1,h)
		glViewport(0, 0, w, h)

	def paintGL(self):
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)
		h = self.size().height()
		w = self.size().width()
		glEnable(GL_DEPTH_TEST)
		glViewport(0, 0, w, h)
		glMatrixMode(GL_PROJECTION)
		glLoadIdentity()
		gluPerspective(45.0, float(w)/float(h), 0.1, 1000000.0)
		glMatrixMode(GL_MODELVIEW)

		glLoadIdentity()					# Reset The View
		glEnable(MeshWidget.GL_MULTISAMPLE)
		glTranslatef(0,0.0,self.zoom)
		glRotate(self.rotation[0], 0.0, 1.0, 0.0)
		glRotate(self.rotation[1], 1.0, 0.0, 0.0)

		center = -self.mesh.bounding_box.center
		glTranslatef(center.x,center.y,center.z)

		if self.draw_mesh:
			if self.enable_restrict:
				for i in self.restricted:
					glBegin(GL_TRIANGLES)
					self.mesh.drawFaceIdx(i)
					glEnd()
			else:
				self.mesh.draw(1.0)
		if self.draw_bounding_box:
			self.mesh.bounding_box.draw()
		if self.draw_octree:
			self.mesh.quad.draw()

		#single vertex marker
		#glPushMatrix(  )
		#s = self.mesh.vertices.verts[self.point_draw]
		#glTranslatef(s.x,s.y,s.z)
		#q = gluNewQuadric()
		#gluSphere( q, GLdouble(0.65), GLint(10), GLint(10) )
		#glPopMatrix(  )

		#self.mesh.drawAdjacentFaces( self.count )
		#self.count += 1
		#if self.count == len(self.mesh.faces):
			#self.count = 0
		#time.sleep(0.3)
		#self.mesh.smooth(0.45)
		w /= 5
		h /= 5
		glViewport(0, 0, w, h)
		glMatrixMode(GL_PROJECTION)
		glLoadIdentity()
		gluPerspective(45.0, float(w)/float(h), 0.1, 1000000.0)
		glMatrixMode(GL_MODELVIEW)
		glDisable(GL_DEPTH_TEST)
		glLoadIdentity()					# Reset The View
		glTranslatef(0,0.0,-self.mesh.bounding_box.minViewDistance())
		glRotate(self.rotation[0], 0.0, 1.0, 0.0)
		glRotate(self.rotation[1], 1.0, 0.0, 0.0)

		center = -self.mesh.bounding_box.center
		glTranslatef(center.x,center.y,center.z)

		if self.draw_mesh:
			self.mesh.draw(0.5)

	def wheelEvent( self, event):
		self.zoom += ( event.delta() / 120 ) * 5
		self.update()

	def mousePressEvent(self, event):
		self.prev_mouse = (event.x(), event.y())

	def mouseMoveEvent(self, event):
		x = event.x()
		y = event.y()

		sensitivity = 0.2
		mouse_x,mouse_y = self.prev_mouse
		self.rotation[0] -= sensitivity*(mouse_x-x)
		self.rotation[1] -= sensitivity*(mouse_y-y)
		self.prev_mouse = (x,y)
		self.update()
		event.accept()

	def __init__(self, parent,filename):
		super(MeshWidget,
				self).__init__(QtOpenGL.QGLFormat(QtOpenGL.QGL.SampleBuffers),
													parent)
		self.setMinimumSize(640, 480)
		self.count = 0
		self.draw_bounding_box = self.draw_octree = False
		self.draw_mesh = True

		self.mesh = mesh.Mesh( 3 )
		self.mesh.parse( filename )
		self.setAutoBufferSwap(True)
		self.initializeGL()
		self.zoom = -self.mesh.bounding_box.minViewDistance()
		self.enable_restrict = False
		self.restricted = range(len(self.mesh.faces))
		self.point_draw = 1

	def reset(self):
		self.rotation=[0,0]
		self.prev_mouse = (0,0)
		self.zoom = -self.mesh.bounding_box.minViewDistance()
		self.update()


class MeshViewer(QtGui.QMainWindow):
	ESCAPE = '\033'

	def __init__(self,filename=None):
		QtGui.QMainWindow.__init__(self)
		self.filename = filename
		self.widget = MeshWidget(self,self.filename)
		self.bid_widget = BidPanel(self, self.widget.mesh)
		self.setCentralWidget(self.widget)
		self.cp = ControlPanel(self,self.bid_widget)
		self.cp.draw_bounding_box.setChecked( self.widget.draw_bounding_box )
		self.cp.draw_mesh.setChecked( self.widget.draw_mesh )
		cpDock = QtGui.QDockWidget("Control Panel", self)
		cpDock.setAllowedAreas(QtCore.Qt.LeftDockWidgetArea |
								QtCore.Qt.RightDockWidgetArea)
		cpDock.setWidget(self.cp)
		self.addDockWidget(0x2,cpDock)

	def keyPressEvent(self, event):
		args = [event.text()]
		# If escape is pressed, kill everything.
		if args[0] == MeshViewer.ESCAPE:
			sys.exit()
		event.accept()

	def smoothLaplace(self):
		for i in range(self.cp.iterationsSpinBox.value()):
			self.widget.mesh.smooth(self.cp.lambdaSpinBox.value())
			self.widget.update()

	def smoothMeanNormal(self):
		for i in range(self.cp.iterationsSpinBox.value()):
			self.widget.mesh.smooth2()
			self.widget.update()

	def reset(self):
		self.cp.setMinMaxSlider()
		self.widget.reset()
		self.widget.update()

	def setOptions(self):
		self.widget.draw_bounding_box = self.cp.draw_bounding_box.isChecked()
		self.widget.draw_mesh = self.cp.draw_mesh.isChecked()
		self.widget.update()

	def noise(self):
		for i in range(self.cp.iterationsSpinBox.value()):
			self.widget.mesh.noise( self.cp.lambdaSpinBox.value() )
			self.widget.update()

	def reload(self):
		self.widget.mesh.parse( self.filename )
		self.widget.mesh.prepDraw()
		self.widget.update()

	def save(self):
		self.filename = str(QtGui.QFileDialog.getSaveFileName(self,
							'Select file to save to'))
		self.filename = self.widget.mesh.write( self.filename )
		self.widget.update()

	def load(self):
		self.filename = str(QtGui.QFileDialog.getOpenFileName(self,
							'Select file to load'))
		self.reload()

	def enable_restrict(self):
		pass

	def set_restrict(self,k):
		self.widget.enable_restrict = self.cp.enable_restrict.isChecked()
		self.widget.restricted = range( self.cp.slider_min.value(),
									 self.cp.slider_max.value() )
		self.widget.update()
		print  self.cp.slider_min.value(), self.cp.slider_max.value()

	def saveScreenshot(self):
		filename = str(QtGui.QFileDialog.getSaveFileName(self,
							'Select file to save to'))
		h = self.widget.size().height()
		w = self.widget.size().width()
		screenshot = glReadPixels( 0,0, w, h, GL_RGBA, GL_UNSIGNED_BYTE)
		Image.frombuffer("RGBA", (w,h), screenshot,
							"raw", "RGBA", 0, 0).save(filename)

	def setPointDraw(self):
		self.widget.point_draw = self.cp.point_num.value()
		self.widget.update()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
tube.py (c) 2010 rene.milk@uni-muenster.de
Licence: WTFPLv2, see LICENSE.txt
"""

from dune.mesh.util.gridhelper import *
from _functors import *
from optparse import OptionParser
import math, copy, sys
from dune.mesh.util.meshutil import BoundaryIdToColorMapper

use_hyperbole_functor = False
def functor_callback(option, opt_str, value, parser):
	global use_hyperbole_functor
	setattr(parser.values, option.dest, 1)
	use_hyperbole_functor = True

def generate():
	global use_hyperbole_functor
	parser = OptionParser()
	parser.add_option("-a", "--area", dest="area", default=math.pi,
					help="area of boundary", type='float')

	parser.add_option("-l", "--length", dest="tube_length", default=10,
					help="tube length", type='float')

	parser.add_option("-y", "--hyp_fac", dest="hyp_fac", default=4.,
					help="hyperbole factor", type='float',
					action='callback',callback=functor_callback )

	parser.add_option("-d", "--hyp_add", dest="hyp_add", default=1.,
					help="hyperbole additive", type='float',
					action='callback',callback=functor_callback )

	parser.add_option("-n", "--num_verts", dest="num_verts", default=6,
					help="number of vertices in circle approx", type='int')

	parser.add_option("-m", "--num_midrings", dest="num_midrings", default=0,
					help="number of midrings inserted between boundary faces", type='int')

	parser.add_option("-s", "--num_spoke_points", dest="num_spoke_points", default=2,
					help="number of spoke points on boundary faces", type='int')

	parser.add_option("-f", "--filename", dest="filename", default='out.smesh',
					help="output filename", type='string')

	(options, args) = parser.parse_args()

	area 			= options.area
	tube_length		= options.tube_length
	num_verts 		= options.num_verts
	alpha			= math.radians( 360. / (num_verts ) )
	alpha_half		= alpha / 2.0
	num_midrings	= int(options.num_midrings)
	num_spoke_points = options.num_spoke_points
	assert num_spoke_points > 0
	area_one_tri = area * math.pi / float(num_verts)
	L_x = math.sqrt( area_one_tri / ( math.sin( alpha_half ) * math.cos( alpha_half ) ) )
	if use_hyperbole_functor:
		functor = HyperboleFunctorZ(tube_length, options.hyp_fac, options.hyp_add )
		print "using hyperbole functor"
	else:
		functor = IdentityFunctor()
		print "using identity functor"

	bidMapper = BoundaryIdToColorMapper(5)
	"""left boundary area"""

	origin_L = Vector3(0,0,0)
	L = functor.scale( Vector3( L_x, 0, 0 ) )
	bound_L = BoundarySurface( bidMapper, origin_L, L, num_spoke_points, num_verts, 2 )

	grid = FullGrid( bound_L, 4 )

	""" midrings, if num > 0 """
	rot_mat = Matrix4.new_rotatez( alpha )
	incr = tube_length / float( num_midrings + 1 )
	for i in range( 1, num_midrings + 1 ):
		points_M = PLCPointList( 3 )
		bound_M = InbetweenRing( )
		M = functor.scale( Vector3( L_x, 0, incr*float(i) ) )
		bound_M.addVertex( points_M.appendVert( M ) )
		for i in range( 1, num_verts  ):
			M = rot_mat  * M
			bound_M.addVertex(points_M.appendVert( M ))
		grid.connect(bound_M)

	"""right boundary"""
	kipp_mat = Matrix4.new_rotatey( math.radians(15) )

	origin_R = Vector3(0,0,tube_length )
	R = functor.scale( Vector3( L_x, 0, tube_length ) )
	bound_R = BoundarySurface( bidMapper, origin_R, R, num_spoke_points, num_verts, 3, True )

	grid.connect(bound_R)
	grid.outputPLC( options.filename, sys.argv, options )



"""
read.py (c) 2010 rene.milk@uni-muenster.de
Licence: WTFPLv2, see LICENSE.txt
"""

try:
	#the spooled variant is new in 2.6
	from tempfile import SpooledTemporaryFile as TemporaryFile
except:
	from tempfile import TemporaryFile

from ..util.meshutil import ( MeshVertexList, 
	Simplex3, BoundaryIdToColorMapper, 
	ColorToBoundaryIdMapper, vector, find_key )
from ..util.euclid import Vector3,Vector2

def _skipCommentsAndEmptyLines(fd):
	while fd:
		line = fd.readline()
		if line.startswith( '#' ):
			continue
		else:
			break
	return fd

def _parseSMESH_vertices(self,fn):
	for line in fn.readlines():
		line = line.split()
		#this way I can use vector for either dim
		line.append(0)
		v = vector( line[1], line[2], line[3] )
		#use a dummy color
		self.vertex_list.addVertex( v, Vector3() )
	print 'read %d vertices'%len(self.vertex_list)

def _parseSMESH_faces(self,fn,bidToColorMapper=BoundaryIdToColorMapper()):
	for line in fn.readlines():
		line = line.split()
		if not self.zero_based_idx:
			v = map( lambda p: self.vertex_list.realIndex(int(p) -1 ), line[1:4] )
		else:
			v = map( lambda p: self.vertex_list.realIndex(int(p) ), line[1:4] )
		boundary_id = int(line[4])
		color = bidToColorMapper.getColor( boundary_id )
		if self.refine:
			assert False
			d0 = self.vertex_list.verts[v0]
			d1 = self.vertex_list.verts[v1]
			d2 = self.vertex_list.verts[v2]
			c = (d0+d1+d2)/3.0
			c_id = self.vertex_list.addVertex( c, color )
			s0 = Simplex3(v0,v1,c_id,self.vertex_list,len(self.faces),boundary_id )
			self.faces.append( s0 )
			s1 = Simplex3(v1,v2,c_id,self.vertex_list,len(self.faces),boundary_id )
			self.faces.append( s1 )
			s2 = Simplex3(v2,v0,c_id,self.vertex_list,len(self.faces),boundary_id )
			self.faces.append( s2 )
		else:
			s = Simplex3(v[0],v[1],v[2],self.vertex_list,len(self.faces),color, boundary_id )
			self.faces.append( s )
			for one_v in v:
				if self.adj_points.has_key(one_v) :
					self.adj_points[one_v] = self.adj_points[one_v].union( set(v) )
				else:
					self.adj_points[one_v] = set(v)

				if self.adj_faces.has_key(one_v):
					self.adj_faces[one_v].append( len(self.faces) - 1 )
				else:
					self.adj_faces[one_v] = [ len(self.faces) - 1 ]
	print 'read %d faces'%len(self.faces)


def parseSMESH(self, filename):
	vert_fn_ = filename + '.vertices'
	face_fn_ = filename + '.faces'
	verts = TemporaryFile(mode='w+r')
	faces = TemporaryFile(mode='w+r')
	fd = open( filename, 'r' )
	fd = _skipCommentsAndEmptyLines( fd )
	#skip one more line..
	dummy = fd.readline()
	first_vert = True
	while fd:
		line = fd.readline()
		if line.startswith( '#' ):
			continue
		if len(line.split()) < self.dim + 0:
			break
		if first_vert:
			self.zero_based_idx = line.startswith('0')
			first_vert = False
		verts.write(line)
	print 'vertice writing complete'

	fd = _skipCommentsAndEmptyLines( fd )
	#skip one more line..
	dummy = fd.readline()
	while fd:
		line = fd.readline()
		if line.startswith( '#' ):
			continue
		if len(line.split()) < self.dim + 1:
			break
		faces.write(line)
	print 'face writing complete'

	#we need the cursor at top for individual parsing
	verts.seek(0)
	faces.seek(0)
	_parseSMESH_vertices(self,verts)
	print 'vert parsing complete'
	_parseSMESH_faces(self, faces)
	print 'face parsing complete'
	self.buildAdjacencyList()



#!/usr/bin/env python
"""
read.py (c) 2011 rene.milk@uni-muenster.de
Licence: WTFPLv2, see LICENSE.txt
"""

from ..util.meshutil import (Simplex3,
	BoundaryIdToColorMapper)
from ..util.euclid import Vector3, Vector2
from ..util import colors
		
def parse(mesh, dgf):
	for vertex in dgf.vertices:
		if mesh.dim == 2:
			vert = Vector2(vertex[0], vertex[1])
		else:
			vert = Vector3(vertex[0], vertex[1], vertex[2])
		mesh.vertex_list.addVertex(vert, colors.constants['white'])
	bid_color_mapper = BoundaryIdToColorMapper()
	for vert in dgf.boundarysegments:
		color = bid_color_mapper.getColor(vert[0])
		simplex = Simplex3(vert[1], vert[2], vert[3], mesh.vertex_list, 
					len(mesh.faces), color, vert[0] )
		mesh.faces.append(simplex)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
colors.py (c) 2009 rene.milk@uni-muenster.de
Licence: WTFPLv2, see LICENSE.txt
"""

from euclid import Vector3

constants = { 'black': Vector3(0,0,0), 'white':Vector3(255,255,255)}

def getHueVector( amount ):
	level = 0;
	while (1 << level) < amount:
		level += 1

	out = []
	return getHueVectorRec(out, amount, level)

def getHueVectorRec( out, amount, level ):
	if level <= 1:
		if len(out) < amount:
			out.append(0.0)
		if len(out) < amount:
			out.append(0.5)
		return out
	else:
		out = getHueVectorRec(out, amount, level - 1);
		lower = len(out)
		out = getHueVectorRec(out, amount, level - 1);
		upper = len(out)
		for i in range(lower,upper):
			out[i] += 1.0 / (1 << level)
		return out

def getColourPalette( size ):
	result = []#colors
	huevector = []#doubles
	satvalbifurcatepos = 0;
	satvalsplittings = []#doubles
	if len(satvalsplittings) == 0:
		#// insert ranges to bifurcate
		satvalsplittings.append( 1 )
		satvalsplittings.append( 0 )
		satvalbifurcatepos = 0

	huevector = getHueVector( size );
	bisectionlimit = 20;
	for i in range( len(result), size ):
		hue = huevector[i];
		saturation = 1;
		value = 1;
		switccolors = i % 3#; // why only 3 and not all combinations? because it's easy, plus the bisection limit cannot be divided integer by it

		if i % bisectionlimit == 0:
			satvalbifurcatepos = satvalbifurcatepos % ( len(satvalsplittings) -1 )
			toinsert = satvalbifurcatepos + 1
			satvalsplittings.insert( toinsert, ( satvalsplittings[satvalbifurcatepos] - satvalsplittings[satvalbifurcatepos + 1] ) / 2 + satvalsplittings[satvalbifurcatepos + 1] )
			satvalbifurcatepos += 2;

		if switccolors == 1:
			saturation = satvalsplittings[satvalbifurcatepos -1]
		elif switccolors == 2 :
			value = satvalsplittings[satvalbifurcatepos -1];

		hue += 0.17#; // use as starting point a zone where color band is narrow so that small variations means high change in visual effect
		if hue > 1:
			hue -= 1
		import colorsys
		col = colorsys.hsv_to_rgb( hue, saturation, value )
		result.append( col )
	return result

def getColourPaletteCheat( size, filter_colors=[] ):
	k = []
	org_size = size
	while len(k) < org_size:
		size += 1
		k = filter(lambda p: p not in filter_colors, set( getColourPalette( size ) ))
	return list(k)

if __name__ == '__main__':
	#k = getColourPalette( 9 )
	#print k
	#k = set(k)
	#print k
	print getColourPaletteCheat(4)
	print getColourPaletteCheat(4,[(0.0,0,0.0)])


#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
randomized.py (c) 2010 rene.milk@uni-muenster.de
Licence: WTFPLv2, see LICENSE.txt
"""


import matplotlib as ml
import matplotlib.pyplot as pp
import matplotlib.delaunay
import random

class RandomGrid( ml.delaunay.triangulate.Triangulation ):
	def __init__( self, num_inserted=4,random_func=random.uniform ):
		min_x = -1
		max_x = 1
		min_y = -1
		max_y = 1
		x = [min_x,min_x,max_x,max_x]
		y = [min_y,max_y,max_y,min_y]
		x.extend( random_func(min_x,max_x) for i in range(num_inserted) )
		y.extend( random_func(min_y,max_y) for i in range(num_inserted) )
		super(RandomGrid,self).__init__( x, y )
		self.xtris = [ [self.x[node[i]] for i in range(3) ] for node in self.triangle_nodes]
		self.ytris = [ [self.y[node[i]] for i in range(3) ] for node in self.triangle_nodes]
       
	def toSmesh(self,filename):
		with open( filename, 'wb' ) as f:
			f.write( '%d 2 0 %d\n'%(len(self.x),1) )
			vert_count = 0
			for x,y in zip(self.x,self.y):
				f.write( '%d %f %f 0\n'%(vert_count,x,y ) )
				vert_count += 1
			f.write( '%d 1\n'%(len(self.triangle_nodes)*3) )            
			line_count = 0
			bid = 1
			for node in self.triangle_nodes:
				f.write( '%d %d %d %d\n'%(line_count,node[0],node[1],bid)  )
				line_count += 1
				f.write( '%d %d %d %d\n'%(line_count,node[1],node[2],bid)  )
				line_count += 1
				f.write( '%d %d %d %d\n'%(line_count,node[2],node[0],bid)  )
				line_count += 1
			f.write( '%d\n'%(0))
			
	def toDGF(self,filename):
		with open( filename, 'wb' ) as f:
			f.write( 'DGF\nVERTEX\n' )
			for x,y in zip(self.x,self.y):
				f.write( '%f %f\n'%(x,y ) )
			f.write( '#\nSIMPLEX\n' )            
			line_count = 0
			for node in self.triangle_nodes:
				f.write( '%d %d %d\n'%(node[0],node[1],node[2])  )
			f.write( 'BOUNDARYDOMAIN\ndefault 5\n#\n')
            
	def tris(self):
		return self.xtris,self.ytris

from optparse import OptionParser

def generate():
	## global defines
	parser = OptionParser()
	parser.add_option("-p", "--points", dest="points", default=100,
					help="points", type='int')
	parser.add_option("-f", "--filename", dest="filename", default="random",
					help="filename", type='string')
	parser.add_option("-c", "--length_c", dest="length_c", default=0.2,
					help="length_c", type='float')
	(options, args) = parser.parse_args()

	if False:
		import matplotlib.gridspec as gridspec
		gs = gridspec.GridSpec(1, 4,width_ratios=[1,1])
		pp.figure(figsize=(12, 12))
		for i in range(4):
			grid = RandomGrid(  )
			#grid.toSmesh("m.smesh")
			xtris,ytris = grid.tris()
			#pp.subplot(1,3,i)
			pp.subplot(gs[0,i])
			for xtips,ytips in zip(xtris,ytris):
				pp.fill(xtips,ytips,facecolor='r',edgecolor='b')
		pp.title('Sequential Plotting')
		pp.show()
	else:
		grid = RandomGrid( options.points, random.triangular )
		grid.toDGF("%s.dgf"%options.filename)
		grid.toSmesh("%s.poly"%options.filename)
		#pp.figure(figsize=(12, 12))
		#xtris,ytris = grid.tris()
		#for xtips,ytips in zip(xtris,ytris):
			#pp.fill(xtips,ytips,facecolor='r',edgecolor='b')
		#pp.show()



#!/usr/bin/env python
"""
mesh.py (c) 2009 rene.milk@uni-muenster.de
Licence: WTFPLv2, see LICENSE.txt
"""


from OpenGL.GL import (glBegin, glVertex3fv, glPushMatrix, glTranslatef,
	glPopMatrix, glEndList, glCallList, glColor4f, glNewList, glEnd,
	GLdouble, glGenLists, GL_COMPILE, GLint, GL_QUADS )
from OpenGL.GLU import (gluNewQuadric, gluSphere)
from dune.mesh.util.gridhelper import adaptVec


class Box:
	def __init__(self,points):
		assert len(points)==8
		self.corners = points[:]
		self.dl = glGenLists(1)
		self.prepDraw()
		
	def prepDraw(self):
		glNewList(self.dl,GL_COMPILE)
		
		c = self.corners
		for h in c:
			glPushMatrix(  )
			glTranslatef(h.x,h.y,h.z)
			q = gluNewQuadric()
			gluSphere( q, GLdouble(0.45), GLint(10), GLint(10) )
			glPopMatrix(  )

		glBegin(GL_QUADS)

		glVertex3fv(adaptVec(c[0]))
		glVertex3fv(adaptVec(c[2]))
		glVertex3fv(adaptVec(c[6]))
		glVertex3fv(adaptVec(c[4]))

		glVertex3fv(adaptVec(c[1]))
		glVertex3fv(adaptVec(c[3]))
		glVertex3fv(adaptVec(c[7]))
		glVertex3fv(adaptVec(c[5]))

		glVertex3fv(adaptVec(c[0]))
		glVertex3fv(adaptVec(c[1]))
		glVertex3fv(adaptVec(c[3]))
		glVertex3fv(adaptVec(c[2]))

		glVertex3fv(adaptVec(c[2]))
		glVertex3fv(adaptVec(c[3]))
		glVertex3fv(adaptVec(c[7]))
		glVertex3fv(adaptVec(c[6]))

		glVertex3fv(adaptVec(c[7]))
		glVertex3fv(adaptVec(c[6]))
		glVertex3fv(adaptVec(c[4]))
		glVertex3fv(adaptVec(c[5]))

		glVertex3fv(adaptVec(c[0]))
		glVertex3fv(adaptVec(c[1]))
		glVertex3fv(adaptVec(c[5]))
		glVertex3fv(adaptVec(c[4]))

		glEnd()
		glEndList()
		
	def draw(self):
		glCallList(self.dl)

	def scale(self,f):
		for h in self.corners:
			h *= f
		self.prepDraw()

	def translate(self,v):
		for h in self.corners:
			h += v
		self.prepDraw()

class Quadtree:
	def __init__(self,mesh):
		self.box = mesh.bounding_box.points
		self.children = []
		p = mesh.bounding_box.points[:]
		b = Box( p )
		v = p[7]
		b.translate( -v )
		b.scale(0.5)
		b.translate( v )
		
		self.children.append( b )
		c = Box(b.corners)
		c.translate( (p[6] - p[7]) )
		self.children.append( c )
		d = Box(c.corners)
		d.translate( p[5] - p[7] - p[1] )
		self.children.append( d )
		e = Box(d.corners)
		e.translate( - p[7] - p[1] )
		self.children.append( e )
		f = Box(e.corners)
		f.translate( - p[5] +p[4] )
		self.children.append( f )
		g = Box(e.corners)
		g.translate( +p[4] - p[0] )
		self.children.append( g )
		h = Box(e.corners)
		h.translate( +p[3] - p[2] )
		self.children.append( h )
		i = Box(e.corners)
		i.translate( - p[7] + p[4] )
		self.children.append( i )

	def draw(self):
		n = float( len(self.children) )
		i = 0
		for b in self.children:
			glColor4f(1-i/n,i/n,0,0.5)
			b.draw()
			i += 1



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# euclid graphics maths module
#
# Copyright (c) 2006 Alex Holkner
# Alex.Holkner@mail.google.com
#
# This library is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License as published by the
# Free Software Foundation; either version 2.1 of the License, or (at your
# option) any later version.
# 
# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License
# for more details.
# 
# You should have received a copy of the GNU Lesser General Public License
# along with this library; if not, write to the Free Software Foundation,
# Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA

'''euclid graphics maths module

Documentation and tests are included in the file "euclid.txt", or online
at http://code.google.com/p/pyeuclid
'''

__docformat__ = 'restructuredtext'
__version__ = '$Id$'
__revision__ = '$Revision$'

import math
import operator
import types

# Some magic here.  If _use_slots is True, the classes will derive from
# object and will define a __slots__ class variable.  If _use_slots is
# False, classes will be old-style and will not define __slots__.
#
# _use_slots = True:   Memory efficient, probably faster in future versions
#                      of Python, "better".
# _use_slots = False:  Ordinary classes, much faster than slots in current
#                      versions of Python (2.4 and 2.5).
_use_slots = True

# If True, allows components of Vector2 and Vector3 to be set via swizzling;
# e.g.  v.xyz = (1, 2, 3).  This is much, much slower than the more verbose
# v.x = 1; v.y = 2; v.z = 3,  and slows down ordinary element setting as
# well.  Recommended setting is False.
_enable_swizzle_set = False

# Requires class to derive from object.
if _enable_swizzle_set:
    _use_slots = True

# Implement _use_slots magic.
class _EuclidMetaclass(type):
    def __new__(cls, name, bases, dct):
        if '__slots__' in dct:
            dct['__getstate__'] = cls._create_getstate(dct['__slots__'])
            dct['__setstate__'] = cls._create_setstate(dct['__slots__'])
        if _use_slots:
            return type.__new__(cls, name, bases + (object,), dct)
        else:
            if '__slots__' in dct:
                del dct['__slots__']
            return types.ClassType.__new__(types.ClassType, name, bases, dct)

    @classmethod
    def _create_getstate(cls, slots):
        def __getstate__(self):
            d = {}
            for slot in slots:
                d[slot] = getattr(self, slot)
            return d
        return __getstate__

    @classmethod
    def _create_setstate(cls, slots):
        def __setstate__(self, state):
            for name, value in state.items():
                setattr(self, name, value)
        return __setstate__

__metaclass__ = _EuclidMetaclass

class Vector2:
    __slots__ = ['x', 'y']

    def __init__(self, x=0, y=0):
        self.x = x
        self.y = y

    def __copy__(self):
        return self.__class__(self.x, self.y)

    copy = __copy__

    def __repr__(self):
        return 'Vector2(%.2f, %.2f)' % (self.x, self.y)

    def __eq__(self, other):
        if isinstance(other, Vector2):
            return self.x == other.x and \
                   self.y == other.y
        else:
            assert hasattr(other, '__len__') and len(other) == 2
            return self.x == other[0] and \
                   self.y == other[1]

    def __neq__(self, other):
        return not self.__eq__(other)

    def __nonzero__(self):
        return self.x != 0 or self.y != 0

    def __len__(self):
        return 2

    def __getitem__(self, key):
        return (self.x, self.y)[key]

    def __setitem__(self, key, value):
        l = [self.x, self.y]
        l[key] = value
        self.x, self.y = l

    def __iter__(self):
        return iter((self.x, self.y))

    def __getattr__(self, name):
        try:
            return tuple([(self.x, self.y)['xy'.index(c)] \
                          for c in name])
        except ValueError:
            raise AttributeError, name

    if _enable_swizzle_set:
        # This has detrimental performance on ordinary setattr as well
        # if enabled
        def __setattr__(self, name, value):
            if len(name) == 1:
                object.__setattr__(self, name, value)
            else:
                try:
                    l = [self.x, self.y]
                    for c, v in map(None, name, value):
                        l['xy'.index(c)] = v
                    self.x, self.y = l
                except ValueError:
                    raise AttributeError, name

    def __add__(self, other):
        if isinstance(other, Vector2):
            # Vector + Vector -> Vector
            # Vector + Point -> Point
            # Point + Point -> Vector
            if self.__class__ is other.__class__:
                _class = Vector2
            else:
                _class = Point2
            return _class(self.x + other.x,
                          self.y + other.y)
        else:
            assert hasattr(other, '__len__') and len(other) == 2
            return Vector2(self.x + other[0],
                           self.y + other[1])
    __radd__ = __add__

    def __iadd__(self, other):
        if isinstance(other, Vector2):
            self.x += other.x
            self.y += other.y
        else:
            self.x += other[0]
            self.y += other[1]
        return self

    def __sub__(self, other):
        if isinstance(other, Vector2):
            # Vector - Vector -> Vector
            # Vector - Point -> Point
            # Point - Point -> Vector
            if self.__class__ is other.__class__:
                _class = Vector2
            else:
                _class = Point2
            return _class(self.x - other.x,
                          self.y - other.y)
        else:
            assert hasattr(other, '__len__') and len(other) == 2
            return Vector2(self.x - other[0],
                           self.y - other[1])

   
    def __rsub__(self, other):
        if isinstance(other, Vector2):
            return Vector2(other.x - self.x,
                           other.y - self.y)
        else:
            assert hasattr(other, '__len__') and len(other) == 2
            return Vector2(other.x - self[0],
                           other.y - self[1])

    def __mul__(self, other):
        assert type(other) in (int, long, float)
        return Vector2(self.x * other,
                       self.y * other)

    __rmul__ = __mul__

    def __imul__(self, other):
        assert type(other) in (int, long, float)
        self.x *= other
        self.y *= other
        return self

    def __div__(self, other):
        assert type(other) in (int, long, float)
        return Vector2(operator.div(self.x, other),
                       operator.div(self.y, other))


    def __rdiv__(self, other):
        assert type(other) in (int, long, float)
        return Vector2(operator.div(other, self.x),
                       operator.div(other, self.y))

    def __floordiv__(self, other):
        assert type(other) in (int, long, float)
        return Vector2(operator.floordiv(self.x, other),
                       operator.floordiv(self.y, other))


    def __rfloordiv__(self, other):
        assert type(other) in (int, long, float)
        return Vector2(operator.floordiv(other, self.x),
                       operator.floordiv(other, self.y))

    def __truediv__(self, other):
        assert type(other) in (int, long, float)
        return Vector2(operator.truediv(self.x, other),
                       operator.truediv(self.y, other))


    def __rtruediv__(self, other):
        assert type(other) in (int, long, float)
        return Vector2(operator.truediv(other, self.x),
                       operator.truediv(other, self.y))
    
    def __neg__(self):
        return Vector2(-self.x,
                        -self.y)

    __pos__ = __copy__
    
    def __abs__(self):
        return math.sqrt(self.x ** 2 + \
                         self.y ** 2)

    magnitude = __abs__

    def magnitude_squared(self):
        return self.x ** 2 + \
               self.y ** 2

    def normalize(self):
        d = self.magnitude()
        if d:
            self.x /= d
            self.y /= d
        return self

    def normalized(self):
        d = self.magnitude()
        if d:
            return Vector2(self.x / d, 
                           self.y / d)
        return self.copy()

    def dot(self, other):
        assert isinstance(other, Vector2)
        return self.x * other.x + \
               self.y * other.y

    def cross(self):
        return Vector2(self.y, -self.x)

    def reflect(self, normal):
        # assume normal is normalized
        assert isinstance(normal, Vector2)
        d = 2 * (self.x * normal.x + self.y * normal.y)
        return Vector2(self.x - d * normal.x,
                       self.y - d * normal.y)

class Vector3:
    __slots__ = ['x', 'y', 'z']

    def __init__(self, x=0, y=0, z=0):
        self.x = x
        self.y = y
        self.z = z

    def __copy__(self):
        return self.__class__(self.x, self.y, self.z)

    copy = __copy__

    def __repr__(self):
        return 'Vector3(%.2f, %.2f, %.2f)' % (self.x,
                                              self.y,
                                              self.z)

    def __eq__(self, other):
        if isinstance(other, Vector3):
            return self.x == other.x and \
                   self.y == other.y and \
                   self.z == other.z
        else:
            assert hasattr(other, '__len__') and len(other) == 3
            return self.x == other[0] and \
                   self.y == other[1] and \
                   self.z == other[2]

    def __neq__(self, other):
        return not self.__eq__(other)

    def __nonzero__(self):
        return self.x != 0 or self.y != 0 or self.z != 0

    def __len__(self):
        return 3

    def __getitem__(self, key):
        return (self.x, self.y, self.z)[key]

    def __setitem__(self, key, value):
        l = [self.x, self.y, self.z]
        l[key] = value
        self.x, self.y, self.z = l

    def __iter__(self):
        return iter((self.x, self.y, self.z))

    def __getattr__(self, name):
        try:
            return tuple([(self.x, self.y, self.z)['xyz'.index(c)] \
                          for c in name])
        except ValueError:
            raise AttributeError, name

    if _enable_swizzle_set:
        # This has detrimental performance on ordinary setattr as well
        # if enabled
        def __setattr__(self, name, value):
            if len(name) == 1:
                object.__setattr__(self, name, value)
            else:
                try:
                    l = [self.x, self.y, self.z]
                    for c, v in map(None, name, value):
                        l['xyz'.index(c)] = v
                    self.x, self.y, self.z = l
                except ValueError:
                    raise AttributeError, name


    def __add__(self, other):
        if isinstance(other, Vector3):
            # Vector + Vector -> Vector
            # Vector + Point -> Point
            # Point + Point -> Vector
            if self.__class__ is other.__class__:
                _class = Vector3
            else:
                _class = Point3
            return _class(self.x + other.x,
                          self.y + other.y,
                          self.z + other.z)
        else:
            assert hasattr(other, '__len__') and len(other) == 3
            return Vector3(self.x + other[0],
                           self.y + other[1],
                           self.z + other[2])
    __radd__ = __add__

    def __iadd__(self, other):
        if isinstance(other, Vector3):
            self.x += other.x
            self.y += other.y
            self.z += other.z
        else:
            self.x += other[0]
            self.y += other[1]
            self.z += other[2]
        return self

    def __sub__(self, other):
        if isinstance(other, Vector3):
            # Vector - Vector -> Vector
            # Vector - Point -> Point
            # Point - Point -> Vector
            if self.__class__ is other.__class__:
                _class = Vector3
            else:
                _class = Point3
            return Vector3(self.x - other.x,
                           self.y - other.y,
                           self.z - other.z)
        else:
            assert hasattr(other, '__len__') and len(other) == 3
            return Vector3(self.x - other[0],
                           self.y - other[1],
                           self.z - other[2])

   
    def __rsub__(self, other):
        if isinstance(other, Vector3):
            return Vector3(other.x - self.x,
                           other.y - self.y,
                           other.z - self.z)
        else:
            assert hasattr(other, '__len__') and len(other) == 3
            return Vector3(other.x - self[0],
                           other.y - self[1],
                           other.z - self[2])

    def __mul__(self, other):
        if isinstance(other, Vector3):
            # TODO component-wise mul/div in-place and on Vector2; docs.
            if self.__class__ is Point3 or other.__class__ is Point3:
                _class = Point3
            else:
                _class = Vector3
            return _class(self.x * other.x,
                          self.y * other.y,
                          self.z * other.z)
        else: 
            assert type(other) in (int, long, float)
            return Vector3(self.x * other,
                           self.y * other,
                           self.z * other)

    __rmul__ = __mul__

    def __imul__(self, other):
        assert type(other) in (int, long, float)
        self.x *= other
        self.y *= other
        self.z *= other
        return self

    def __div__(self, other):
        assert type(other) in (int, long, float)
        return Vector3(operator.div(self.x, other),
                       operator.div(self.y, other),
                       operator.div(self.z, other))


    def __rdiv__(self, other):
        assert type(other) in (int, long, float)
        return Vector3(operator.div(other, self.x),
                       operator.div(other, self.y),
                       operator.div(other, self.z))

    def __floordiv__(self, other):
        assert type(other) in (int, long, float)
        return Vector3(operator.floordiv(self.x, other),
                       operator.floordiv(self.y, other),
                       operator.floordiv(self.z, other))


    def __rfloordiv__(self, other):
        assert type(other) in (int, long, float)
        return Vector3(operator.floordiv(other, self.x),
                       operator.floordiv(other, self.y),
                       operator.floordiv(other, self.z))

    def __truediv__(self, other):
        assert type(other) in (int, long, float)
        return Vector3(operator.truediv(self.x, other),
                       operator.truediv(self.y, other),
                       operator.truediv(self.z, other))


    def __rtruediv__(self, other):
        assert type(other) in (int, long, float)
        return Vector3(operator.truediv(other, self.x),
                       operator.truediv(other, self.y),
                       operator.truediv(other, self.z))
    
    def __neg__(self):
        return Vector3(-self.x,
                        -self.y,
                        -self.z)

    __pos__ = __copy__
    
    def __abs__(self):
        return math.sqrt(self.x ** 2 + \
                         self.y ** 2 + \
                         self.z ** 2)

    magnitude = __abs__

    def magnitude_squared(self):
        return self.x ** 2 + \
               self.y ** 2 + \
               self.z ** 2

    def normalize(self):
        d = self.magnitude()
        if d:
            self.x /= d
            self.y /= d
            self.z /= d
        return self

    def normalized(self):
        d = self.magnitude()
        if d:
            return Vector3(self.x / d, 
                           self.y / d, 
                           self.z / d)
        return self.copy()

    def dot(self, other):
        assert isinstance(other, Vector3)
        return self.x * other.x + \
               self.y * other.y + \
               self.z * other.z

    def cross(self, other):
        assert isinstance(other, Vector3)
        return Vector3(self.y * other.z - self.z * other.y,
                       -self.x * other.z + self.z * other.x,
                       self.x * other.y - self.y * other.x)

    def reflect(self, normal):
        # assume normal is normalized
        assert isinstance(normal, Vector3)
        d = 2 * (self.x * normal.x + self.y * normal.y + self.z * normal.z)
        return Vector3(self.x - d * normal.x,
                       self.y - d * normal.y,
                       self.z - d * normal.z)

    def __call__(s):
        return [s.x,s.y,s.z]

# a b c 
# e f g 
# i j k 

class Matrix3:
    __slots__ = list('abcefgijk')

    def __init__(self):
        self.identity()

    def __copy__(self):
        M = Matrix3()
        M.a = self.a
        M.b = self.b
        M.c = self.c
        M.e = self.e 
        M.f = self.f
        M.g = self.g
        M.i = self.i
        M.j = self.j
        M.k = self.k
        return M

    copy = __copy__
    def __repr__(self):
        return ('Matrix3([% 8.2f % 8.2f % 8.2f\n'  \
                '         % 8.2f % 8.2f % 8.2f\n'  \
                '         % 8.2f % 8.2f % 8.2f])') \
                % (self.a, self.b, self.c,
                   self.e, self.f, self.g,
                   self.i, self.j, self.k)

    def __getitem__(self, key):
        return [self.a, self.e, self.i,
                self.b, self.f, self.j,
                self.c, self.g, self.k][key]

    def __setitem__(self, key, value):
        L = self[:]
        L[key] = value
        (self.a, self.e, self.i,
         self.b, self.f, self.j,
         self.c, self.g, self.k) = L

    def __mul__(self, other):
        if isinstance(other, Matrix3):
            # Caching repeatedly accessed attributes in local variables
            # apparently increases performance by 20%.  Attrib: Will McGugan.
            Aa = self.a
            Ab = self.b
            Ac = self.c
            Ae = self.e
            Af = self.f
            Ag = self.g
            Ai = self.i
            Aj = self.j
            Ak = self.k
            Ba = other.a
            Bb = other.b
            Bc = other.c
            Be = other.e
            Bf = other.f
            Bg = other.g
            Bi = other.i
            Bj = other.j
            Bk = other.k
            C = Matrix3()
            C.a = Aa * Ba + Ab * Be + Ac * Bi
            C.b = Aa * Bb + Ab * Bf + Ac * Bj
            C.c = Aa * Bc + Ab * Bg + Ac * Bk
            C.e = Ae * Ba + Af * Be + Ag * Bi
            C.f = Ae * Bb + Af * Bf + Ag * Bj
            C.g = Ae * Bc + Af * Bg + Ag * Bk
            C.i = Ai * Ba + Aj * Be + Ak * Bi
            C.j = Ai * Bb + Aj * Bf + Ak * Bj
            C.k = Ai * Bc + Aj * Bg + Ak * Bk
            return C
        elif isinstance(other, Point2):
            A = self
            B = other
            P = Point2(0, 0)
            P.x = A.a * B.x + A.b * B.y + A.c
            P.y = A.e * B.x + A.f * B.y + A.g
            return P
        elif isinstance(other, Vector2):
            A = self
            B = other
            V = Vector2(0, 0)
            V.x = A.a * B.x + A.b * B.y 
            V.y = A.e * B.x + A.f * B.y 
            return V
        else:
            other = other.copy()
            other._apply_transform(self)
            return other

    def __imul__(self, other):
        assert isinstance(other, Matrix3)
        # Cache attributes in local vars (see Matrix3.__mul__).
        Aa = self.a
        Ab = self.b
        Ac = self.c
        Ae = self.e
        Af = self.f
        Ag = self.g
        Ai = self.i
        Aj = self.j
        Ak = self.k
        Ba = other.a
        Bb = other.b
        Bc = other.c
        Be = other.e
        Bf = other.f
        Bg = other.g
        Bi = other.i
        Bj = other.j
        Bk = other.k
        self.a = Aa * Ba + Ab * Be + Ac * Bi
        self.b = Aa * Bb + Ab * Bf + Ac * Bj
        self.c = Aa * Bc + Ab * Bg + Ac * Bk
        self.e = Ae * Ba + Af * Be + Ag * Bi
        self.f = Ae * Bb + Af * Bf + Ag * Bj
        self.g = Ae * Bc + Af * Bg + Ag * Bk
        self.i = Ai * Ba + Aj * Be + Ak * Bi
        self.j = Ai * Bb + Aj * Bf + Ak * Bj
        self.k = Ai * Bc + Aj * Bg + Ak * Bk
        return self

    def identity(self):
        self.a = self.f = self.k = 1.
        self.b = self.c = self.e = self.g = self.i = self.j = 0
        return self

    def scale(self, x, y):
        self *= Matrix3.new_scale(x, y)
        return self

    def translate(self, x, y):
        self *= Matrix3.new_translate(x, y)
        return self 

    def rotate(self, angle):
        self *= Matrix3.new_rotate(angle)
        return self

    # Static constructors
    def new_identity(cls):
        self = cls()
        return self
    new_identity = classmethod(new_identity)

    def new_scale(cls, x, y):
        self = cls()
        self.a = x
        self.f = y
        return self
    new_scale = classmethod(new_scale)

    def new_translate(cls, x, y):
        self = cls()
        self.c = x
        self.g = y
        return self
    new_translate = classmethod(new_translate)

    def new_rotate(cls, angle):
        self = cls()
        s = math.sin(angle)
        c = math.cos(angle)
        self.a = self.f = c
        self.b = -s
        self.e = s
        return self
    new_rotate = classmethod(new_rotate)

# a b c d
# e f g h
# i j k l
# m n o p

class Matrix4:
    __slots__ = list('abcdefghijklmnop')

    def __init__(self):
        self.identity()

    def __copy__(self):
        M = Matrix4()
        M.a = self.a
        M.b = self.b
        M.c = self.c
        M.d = self.d
        M.e = self.e 
        M.f = self.f
        M.g = self.g
        M.h = self.h
        M.i = self.i
        M.j = self.j
        M.k = self.k
        M.l = self.l
        M.m = self.m
        M.n = self.n
        M.o = self.o
        M.p = self.p
        return M

    copy = __copy__


    def __repr__(self):
        return ('Matrix4([% 8.2f % 8.2f % 8.2f % 8.2f\n'  \
                '         % 8.2f % 8.2f % 8.2f % 8.2f\n'  \
                '         % 8.2f % 8.2f % 8.2f % 8.2f\n'  \
                '         % 8.2f % 8.2f % 8.2f % 8.2f])') \
                % (self.a, self.b, self.c, self.d,
                   self.e, self.f, self.g, self.h,
                   self.i, self.j, self.k, self.l,
                   self.m, self.n, self.o, self.p)

    def __getitem__(self, key):
        return [self.a, self.e, self.i, self.m,
                self.b, self.f, self.j, self.n,
                self.c, self.g, self.k, self.o,
                self.d, self.h, self.l, self.p][key]

    def __setitem__(self, key, value):
        L = self[:]
        L[key] = value
        (self.a, self.e, self.i, self.m,
         self.b, self.f, self.j, self.n,
         self.c, self.g, self.k, self.o,
         self.d, self.h, self.l, self.p) = L

    def __mul__(self, other):
        if isinstance(other, Matrix4):
            # Cache attributes in local vars (see Matrix3.__mul__).
            Aa = self.a
            Ab = self.b
            Ac = self.c
            Ad = self.d
            Ae = self.e
            Af = self.f
            Ag = self.g
            Ah = self.h
            Ai = self.i
            Aj = self.j
            Ak = self.k
            Al = self.l
            Am = self.m
            An = self.n
            Ao = self.o
            Ap = self.p
            Ba = other.a
            Bb = other.b
            Bc = other.c
            Bd = other.d
            Be = other.e
            Bf = other.f
            Bg = other.g
            Bh = other.h
            Bi = other.i
            Bj = other.j
            Bk = other.k
            Bl = other.l
            Bm = other.m
            Bn = other.n
            Bo = other.o
            Bp = other.p
            C = Matrix4()
            C.a = Aa * Ba + Ab * Be + Ac * Bi + Ad * Bm
            C.b = Aa * Bb + Ab * Bf + Ac * Bj + Ad * Bn
            C.c = Aa * Bc + Ab * Bg + Ac * Bk + Ad * Bo
            C.d = Aa * Bd + Ab * Bh + Ac * Bl + Ad * Bp
            C.e = Ae * Ba + Af * Be + Ag * Bi + Ah * Bm
            C.f = Ae * Bb + Af * Bf + Ag * Bj + Ah * Bn
            C.g = Ae * Bc + Af * Bg + Ag * Bk + Ah * Bo
            C.h = Ae * Bd + Af * Bh + Ag * Bl + Ah * Bp
            C.i = Ai * Ba + Aj * Be + Ak * Bi + Al * Bm
            C.j = Ai * Bb + Aj * Bf + Ak * Bj + Al * Bn
            C.k = Ai * Bc + Aj * Bg + Ak * Bk + Al * Bo
            C.l = Ai * Bd + Aj * Bh + Ak * Bl + Al * Bp
            C.m = Am * Ba + An * Be + Ao * Bi + Ap * Bm
            C.n = Am * Bb + An * Bf + Ao * Bj + Ap * Bn
            C.o = Am * Bc + An * Bg + Ao * Bk + Ap * Bo
            C.p = Am * Bd + An * Bh + Ao * Bl + Ap * Bp
            return C
        elif isinstance(other, Point3):
            A = self
            B = other
            P = Point3(0, 0, 0)
            P.x = A.a * B.x + A.b * B.y + A.c * B.z + A.d
            P.y = A.e * B.x + A.f * B.y + A.g * B.z + A.h
            P.z = A.i * B.x + A.j * B.y + A.k * B.z + A.l
            return P
        elif isinstance(other, Vector3):
            A = self
            B = other
            V = Vector3(0, 0, 0)
            V.x = A.a * B.x + A.b * B.y + A.c * B.z
            V.y = A.e * B.x + A.f * B.y + A.g * B.z
            V.z = A.i * B.x + A.j * B.y + A.k * B.z
            return V
        else:
            other = other.copy()
            other._apply_transform(self)
            return other

    def __imul__(self, other):
        assert isinstance(other, Matrix4)
        # Cache attributes in local vars (see Matrix3.__mul__).
        Aa = self.a
        Ab = self.b
        Ac = self.c
        Ad = self.d
        Ae = self.e
        Af = self.f
        Ag = self.g
        Ah = self.h
        Ai = self.i
        Aj = self.j
        Ak = self.k
        Al = self.l
        Am = self.m
        An = self.n
        Ao = self.o
        Ap = self.p
        Ba = other.a
        Bb = other.b
        Bc = other.c
        Bd = other.d
        Be = other.e
        Bf = other.f
        Bg = other.g
        Bh = other.h
        Bi = other.i
        Bj = other.j
        Bk = other.k
        Bl = other.l
        Bm = other.m
        Bn = other.n
        Bo = other.o
        Bp = other.p
        self.a = Aa * Ba + Ab * Be + Ac * Bi + Ad * Bm
        self.b = Aa * Bb + Ab * Bf + Ac * Bj + Ad * Bn
        self.c = Aa * Bc + Ab * Bg + Ac * Bk + Ad * Bo
        self.d = Aa * Bd + Ab * Bh + Ac * Bl + Ad * Bp
        self.e = Ae * Ba + Af * Be + Ag * Bi + Ah * Bm
        self.f = Ae * Bb + Af * Bf + Ag * Bj + Ah * Bn
        self.g = Ae * Bc + Af * Bg + Ag * Bk + Ah * Bo
        self.h = Ae * Bd + Af * Bh + Ag * Bl + Ah * Bp
        self.i = Ai * Ba + Aj * Be + Ak * Bi + Al * Bm
        self.j = Ai * Bb + Aj * Bf + Ak * Bj + Al * Bn
        self.k = Ai * Bc + Aj * Bg + Ak * Bk + Al * Bo
        self.l = Ai * Bd + Aj * Bh + Ak * Bl + Al * Bp
        self.m = Am * Ba + An * Be + Ao * Bi + Ap * Bm
        self.n = Am * Bb + An * Bf + Ao * Bj + Ap * Bn
        self.o = Am * Bc + An * Bg + Ao * Bk + Ap * Bo
        self.p = Am * Bd + An * Bh + Ao * Bl + Ap * Bp
        return self

    def transform(self, other):
        A = self
        B = other
        P = Point3(0, 0, 0)
        P.x = A.a * B.x + A.b * B.y + A.c * B.z + A.d
        P.y = A.e * B.x + A.f * B.y + A.g * B.z + A.h
        P.z = A.i * B.x + A.j * B.y + A.k * B.z + A.l
        w =   A.m * B.x + A.n * B.y + A.o * B.z + A.p
        if w != 0:
            P.x /= w
            P.y /= w
            P.z /= w
        return P

    def identity(self):
        self.a = self.f = self.k = self.p = 1.
        self.b = self.c = self.d = self.e = self.g = self.h = \
        self.i = self.j = self.l = self.m = self.n = self.o = 0
        return self

    def scale(self, x, y, z):
        self *= Matrix4.new_scale(x, y, z)
        return self

    def translate(self, x, y, z):
        self *= Matrix4.new_translate(x, y, z)
        return self 

    def rotatex(self, angle):
        self *= Matrix4.new_rotatex(angle)
        return self

    def rotatey(self, angle):
        self *= Matrix4.new_rotatey(angle)
        return self

    def rotatez(self, angle):
        self *= Matrix4.new_rotatez(angle)
        return self

    def rotate_axis(self, angle, axis):
        self *= Matrix4.new_rotate_axis(angle, axis)
        return self

    def rotate_euler(self, heading, attitude, bank):
        self *= Matrix4.new_rotate_euler(heading, attitude, bank)
        return self

    def rotate_triple_axis(self, x, y, z):
        self *= Matrix4.new_rotate_triple_axis(x, y, z)
        return self

    def transpose(self):
        (self.a, self.e, self.i, self.m,
         self.b, self.f, self.j, self.n,
         self.c, self.g, self.k, self.o,
         self.d, self.h, self.l, self.p) = \
        (self.a, self.b, self.c, self.d,
         self.e, self.f, self.g, self.h,
         self.i, self.j, self.k, self.l,
         self.m, self.n, self.o, self.p)

    def transposed(self):
        M = self.copy()
        M.transpose()
        return M

    # Static constructors
    def new(cls, *values):
        M = cls()
        M[:] = values
        return M
    new = classmethod(new)

    def new_identity(cls):
        self = cls()
        return self
    new_identity = classmethod(new_identity)

    def new_scale(cls, x, y, z):
        self = cls()
        self.a = x
        self.f = y
        self.k = z
        return self
    new_scale = classmethod(new_scale)

    def new_translate(cls, x, y, z):
        self = cls()
        self.d = x
        self.h = y
        self.l = z
        return self
    new_translate = classmethod(new_translate)

    def new_rotatex(cls, angle):
        self = cls()
        s = math.sin(angle)
        c = math.cos(angle)
        self.f = self.k = c
        self.g = -s
        self.j = s
        return self
    new_rotatex = classmethod(new_rotatex)

    def new_rotatey(cls, angle):
        self = cls()
        s = math.sin(angle)
        c = math.cos(angle)
        self.a = self.k = c
        self.c = s
        self.i = -s
        return self    
    new_rotatey = classmethod(new_rotatey)
    
    def new_rotatez(cls, angle):
        self = cls()
        s = math.sin(angle)
        c = math.cos(angle)
        self.a = self.f = c
        self.b = -s
        self.e = s
        return self
    new_rotatez = classmethod(new_rotatez)

    def new_rotate_axis(cls, angle, axis):
        assert(isinstance(axis, Vector3))
        vector = axis.normalized()
        x = vector.x
        y = vector.y
        z = vector.z

        self = cls()
        s = math.sin(angle)
        c = math.cos(angle)
        c1 = 1. - c
        
        # from the glRotate man page
        self.a = x * x * c1 + c
        self.b = x * y * c1 - z * s
        self.c = x * z * c1 + y * s
        self.e = y * x * c1 + z * s
        self.f = y * y * c1 + c
        self.g = y * z * c1 - x * s
        self.i = x * z * c1 - y * s
        self.j = y * z * c1 + x * s
        self.k = z * z * c1 + c
        return self
    new_rotate_axis = classmethod(new_rotate_axis)

    def new_rotate_euler(cls, heading, attitude, bank):
        # from http://www.euclideanspace.com/
        ch = math.cos(heading)
        sh = math.sin(heading)
        ca = math.cos(attitude)
        sa = math.sin(attitude)
        cb = math.cos(bank)
        sb = math.sin(bank)

        self = cls()
        self.a = ch * ca
        self.b = sh * sb - ch * sa * cb
        self.c = ch * sa * sb + sh * cb
        self.e = sa
        self.f = ca * cb
        self.g = -ca * sb
        self.i = -sh * ca
        self.j = sh * sa * cb + ch * sb
        self.k = -sh * sa * sb + ch * cb
        return self
    new_rotate_euler = classmethod(new_rotate_euler)

    def new_rotate_triple_axis(cls, x, y, z):
      m = cls()
      
      m.a, m.b, m.c = x.x, y.x, z.x
      m.e, m.f, m.g = x.y, y.y, z.y
      m.i, m.j, m.k = x.z, y.z, z.z
      
      return m
    new_rotate_triple_axis = classmethod(new_rotate_triple_axis)

    def new_look_at(cls, eye, at, up):
      z = (eye - at).normalized()
      x = up.cross(z).normalized()
      y = z.cross(x)
      
      m = cls.new_rotate_triple_axis(x, y, z)
      m.d, m.h, m.l = eye.x, eye.y, eye.z
      return m
    new_look_at = classmethod(new_look_at)
    
    def new_perspective(cls, fov_y, aspect, near, far):
        # from the gluPerspective man page
        f = 1 / math.tan(fov_y / 2)
        self = cls()
        assert near != 0.0 and near != far
        self.a = f / aspect
        self.f = f
        self.k = (far + near) / (near - far)
        self.l = 2 * far * near / (near - far)
        self.o = -1
        self.p = 0
        return self
    new_perspective = classmethod(new_perspective)

    def determinant(self):
        return ((self.a * self.f - self.e * self.b)
              * (self.k * self.p - self.o * self.l)
              - (self.a * self.j - self.i * self.b)
              * (self.g * self.p - self.o * self.h)
              + (self.a * self.n - self.m * self.b)
              * (self.g * self.l - self.k * self.h)
              + (self.e * self.j - self.i * self.f)
              * (self.c * self.p - self.o * self.d)
              - (self.e * self.n - self.m * self.f)
              * (self.c * self.l - self.k * self.d)
              + (self.i * self.n - self.m * self.j)
              * (self.c * self.h - self.g * self.d))

    def inverse(self):
        tmp = Matrix4()
        d = self.determinant();

        if abs(d) < 0.001:
            # No inverse, return identity
            return tmp
        else:
            d = 1.0 / d;

            tmp.a = d * (self.f * (self.k * self.p - self.o * self.l) + self.j * (self.o * self.h - self.g * self.p) + self.n * (self.g * self.l - self.k * self.h));
            tmp.e = d * (self.g * (self.i * self.p - self.m * self.l) + self.k * (self.m * self.h - self.e * self.p) + self.o * (self.e * self.l - self.i * self.h));
            tmp.i = d * (self.h * (self.i * self.n - self.m * self.j) + self.l * (self.m * self.f - self.e * self.n) + self.p * (self.e * self.j - self.i * self.f));
            tmp.m = d * (self.e * (self.n * self.k - self.j * self.o) + self.i * (self.f * self.o - self.n * self.g) + self.m * (self.j * self.g - self.f * self.k));
            
            tmp.b = d * (self.j * (self.c * self.p - self.o * self.d) + self.n * (self.k * self.d - self.c * self.l) + self.b * (self.o * self.l - self.k * self.p));
            tmp.f = d * (self.k * (self.a * self.p - self.m * self.d) + self.o * (self.i * self.d - self.a * self.l) + self.c * (self.m * self.l - self.i * self.p));
            tmp.j = d * (self.l * (self.a * self.n - self.m * self.b) + self.p * (self.i * self.b - self.a * self.j) + self.d * (self.m * self.j - self.i * self.n));
            tmp.n = d * (self.i * (self.n * self.c - self.b * self.o) + self.m * (self.b * self.k - self.j * self.c) + self.a * (self.j * self.o - self.n * self.k));
            
            tmp.c = d * (self.n * (self.c * self.h - self.g * self.d) + self.b * (self.g * self.p - self.o * self.h) + self.f * (self.o * self.d - self.c * self.p));
            tmp.g = d * (self.o * (self.a * self.h - self.e * self.d) + self.c * (self.e * self.p - self.m * self.h) + self.g * (self.m * self.d - self.a * self.p));
            tmp.k = d * (self.p * (self.a * self.f - self.e * self.b) + self.d * (self.e * self.n - self.m * self.f) + self.h * (self.m * self.b - self.a * self.n));
            tmp.o = d * (self.m * (self.f * self.c - self.b * self.g) + self.a * (self.n * self.g - self.f * self.o) + self.e * (self.b * self.o - self.n * self.c));
            
            tmp.d = d * (self.b * (self.k * self.h - self.g * self.l) + self.f * (self.c * self.l - self.k * self.d) + self.j * (self.g * self.d - self.c * self.h));
            tmp.h = d * (self.c * (self.i * self.h - self.e * self.l) + self.g * (self.a * self.l - self.i * self.d) + self.k * (self.e * self.d - self.a * self.h));
            tmp.l = d * (self.d * (self.i * self.f - self.e * self.j) + self.h * (self.a * self.j - self.i * self.b) + self.l * (self.e * self.b - self.a * self.f));
            tmp.p = d * (self.a * (self.f * self.k - self.j * self.g) + self.e * (self.j * self.c - self.b * self.k) + self.i * (self.b * self.g - self.f * self.c));

        return tmp;
        

class Quaternion:
    # All methods and naming conventions based off 
    # http://www.euclideanspace.com/maths/algebra/realNormedAlgebra/quaternions

    # w is the real part, (x, y, z) are the imaginary parts
    __slots__ = ['w', 'x', 'y', 'z']

    def __init__(self, w=1, x=0, y=0, z=0):
        self.w = w
        self.x = x
        self.y = y
        self.z = z

    def __copy__(self):
        Q = Quaternion()
        Q.w = self.w
        Q.x = self.x
        Q.y = self.y
        Q.z = self.z
        return Q

    copy = __copy__

    def __repr__(self):
        return 'Quaternion(real=%.2f, imag=<%.2f, %.2f, %.2f>)' % \
            (self.w, self.x, self.y, self.z)

    def __mul__(self, other):
        if isinstance(other, Quaternion):
            Ax = self.x
            Ay = self.y
            Az = self.z
            Aw = self.w
            Bx = other.x
            By = other.y
            Bz = other.z
            Bw = other.w
            Q = Quaternion()
            Q.x =  Ax * Bw + Ay * Bz - Az * By + Aw * Bx    
            Q.y = -Ax * Bz + Ay * Bw + Az * Bx + Aw * By
            Q.z =  Ax * By - Ay * Bx + Az * Bw + Aw * Bz
            Q.w = -Ax * Bx - Ay * By - Az * Bz + Aw * Bw
            return Q
        elif isinstance(other, Vector3):
            w = self.w
            x = self.x
            y = self.y
            z = self.z
            Vx = other.x
            Vy = other.y
            Vz = other.z
            return other.__class__(\
               w * w * Vx + 2 * y * w * Vz - 2 * z * w * Vy + \
               x * x * Vx + 2 * y * x * Vy + 2 * z * x * Vz - \
               z * z * Vx - y * y * Vx,
               2 * x * y * Vx + y * y * Vy + 2 * z * y * Vz + \
               2 * w * z * Vx - z * z * Vy + w * w * Vy - \
               2 * x * w * Vz - x * x * Vy,
               2 * x * z * Vx + 2 * y * z * Vy + \
               z * z * Vz - 2 * w * y * Vx - y * y * Vz + \
               2 * w * x * Vy - x * x * Vz + w * w * Vz)
        else:
            other = other.copy()
            other._apply_transform(self)
            return other

    def __imul__(self, other):
        assert isinstance(other, Quaternion)
        Ax = self.x
        Ay = self.y
        Az = self.z
        Aw = self.w
        Bx = other.x
        By = other.y
        Bz = other.z
        Bw = other.w
        self.x =  Ax * Bw + Ay * Bz - Az * By + Aw * Bx    
        self.y = -Ax * Bz + Ay * Bw + Az * Bx + Aw * By
        self.z =  Ax * By - Ay * Bx + Az * Bw + Aw * Bz
        self.w = -Ax * Bx - Ay * By - Az * Bz + Aw * Bw
        return self

    def __abs__(self):
        return math.sqrt(self.w ** 2 + \
                         self.x ** 2 + \
                         self.y ** 2 + \
                         self.z ** 2)

    magnitude = __abs__

    def magnitude_squared(self):
        return self.w ** 2 + \
               self.x ** 2 + \
               self.y ** 2 + \
               self.z ** 2 

    def identity(self):
        self.w = 1
        self.x = 0
        self.y = 0
        self.z = 0
        return self

    def rotate_axis(self, angle, axis):
        self *= Quaternion.new_rotate_axis(angle, axis)
        return self

    def rotate_euler(self, heading, attitude, bank):
        self *= Quaternion.new_rotate_euler(heading, attitude, bank)
        return self

    def rotate_matrix(self, m):
        self *= Quaternion.new_rotate_matrix(m)
        return self

    def conjugated(self):
        Q = Quaternion()
        Q.w = self.w
        Q.x = -self.x
        Q.y = -self.y
        Q.z = -self.z
        return Q

    def normalize(self):
        d = self.magnitude()
        if d != 0:
            self.w /= d
            self.x /= d
            self.y /= d
            self.z /= d
        return self

    def normalized(self):
        d = self.magnitude()
        if d != 0:
            Q = Quaternion()
            Q.w = self.w / d
            Q.x = self.x / d
            Q.y = self.y / d
            Q.z = self.z / d
            return Q
        else:
            return self.copy()

    def get_angle_axis(self):
        if self.w > 1:
            self = self.normalized()
        angle = 2 * math.acos(self.w)
        s = math.sqrt(1 - self.w ** 2)
        if s < 0.001:
            return angle, Vector3(1, 0, 0)
        else:
            return angle, Vector3(self.x / s, self.y / s, self.z / s)

    def get_euler(self):
        t = self.x * self.y + self.z * self.w
        if t > 0.4999:
            heading = 2 * math.atan2(self.x, self.w)
            attitude = math.pi / 2
            bank = 0
        elif t < -0.4999:
            heading = -2 * math.atan2(self.x, self.w)
            attitude = -math.pi / 2
            bank = 0
        else:
            sqx = self.x ** 2
            sqy = self.y ** 2
            sqz = self.z ** 2
            heading = math.atan2(2 * self.y * self.w - 2 * self.x * self.z,
                                 1 - 2 * sqy - 2 * sqz)
            attitude = math.asin(2 * t)
            bank = math.atan2(2 * self.x * self.w - 2 * self.y * self.z,
                              1 - 2 * sqx - 2 * sqz)
        return heading, attitude, bank

    def get_matrix(self):
        xx = self.x ** 2
        xy = self.x * self.y
        xz = self.x * self.z
        xw = self.x * self.w
        yy = self.y ** 2
        yz = self.y * self.z
        yw = self.y * self.w
        zz = self.z ** 2
        zw = self.z * self.w
        M = Matrix4()
        M.a = 1 - 2 * (yy + zz)
        M.b = 2 * (xy - zw)
        M.c = 2 * (xz + yw)
        M.e = 2 * (xy + zw)
        M.f = 1 - 2 * (xx + zz)
        M.g = 2 * (yz - xw)
        M.i = 2 * (xz - yw)
        M.j = 2 * (yz + xw)
        M.k = 1 - 2 * (xx + yy)
        return M

    # Static constructors
    def new_identity(cls):
        return cls()
    new_identity = classmethod(new_identity)

    def new_rotate_axis(cls, angle, axis):
        assert(isinstance(axis, Vector3))
        axis = axis.normalized()
        s = math.sin(angle / 2)
        Q = cls()
        Q.w = math.cos(angle / 2)
        Q.x = axis.x * s
        Q.y = axis.y * s
        Q.z = axis.z * s
        return Q
    new_rotate_axis = classmethod(new_rotate_axis)

    def new_rotate_euler(cls, heading, attitude, bank):
        Q = cls()
        c1 = math.cos(heading / 2)
        s1 = math.sin(heading / 2)
        c2 = math.cos(attitude / 2)
        s2 = math.sin(attitude / 2)
        c3 = math.cos(bank / 2)
        s3 = math.sin(bank / 2)

        Q.w = c1 * c2 * c3 - s1 * s2 * s3
        Q.x = s1 * s2 * c3 + c1 * c2 * s3
        Q.y = s1 * c2 * c3 + c1 * s2 * s3
        Q.z = c1 * s2 * c3 - s1 * c2 * s3
        return Q
    new_rotate_euler = classmethod(new_rotate_euler)
    
    def new_rotate_matrix(cls, m):
      if m[0*4 + 0] + m[1*4 + 1] + m[2*4 + 2] > 0.00000001:
        t = m[0*4 + 0] + m[1*4 + 1] + m[2*4 + 2] + 1.0
        s = 0.5/math.sqrt(t)
        
        return cls(
          s*t,
          (m[1*4 + 2] - m[2*4 + 1])*s,
          (m[2*4 + 0] - m[0*4 + 2])*s,
          (m[0*4 + 1] - m[1*4 + 0])*s
          )
        
      elif m[0*4 + 0] > m[1*4 + 1] and m[0*4 + 0] > m[2*4 + 2]:
        t = m[0*4 + 0] - m[1*4 + 1] - m[2*4 + 2] + 1.0
        s = 0.5/math.sqrt(t)
        
        return cls(
          (m[1*4 + 2] - m[2*4 + 1])*s,
          s*t,
          (m[0*4 + 1] + m[1*4 + 0])*s,
          (m[2*4 + 0] + m[0*4 + 2])*s
          )
        
      elif m[1*4 + 1] > m[2*4 + 2]:
        t = -m[0*4 + 0] + m[1*4 + 1] - m[2*4 + 2] + 1.0
        s = 0.5/math.sqrt(t)
        
        return cls(
          (m[2*4 + 0] - m[0*4 + 2])*s,
          (m[0*4 + 1] + m[1*4 + 0])*s,
          s*t,
          (m[1*4 + 2] + m[2*4 + 1])*s
          )
        
      else:
        t = -m[0*4 + 0] - m[1*4 + 1] + m[2*4 + 2] + 1.0
        s = 0.5/math.sqrt(t)
        
        return cls(
          (m[0*4 + 1] - m[1*4 + 0])*s,
          (m[2*4 + 0] + m[0*4 + 2])*s,
          (m[1*4 + 2] + m[2*4 + 1])*s,
          s*t
          )
    new_rotate_matrix = classmethod(new_rotate_matrix)
    
    def new_interpolate(cls, q1, q2, t):
        assert isinstance(q1, Quaternion) and isinstance(q2, Quaternion)
        Q = cls()

        costheta = q1.w * q2.w + q1.x * q2.x + q1.y * q2.y + q1.z * q2.z
        if costheta < 0.:
            costheta = -costheta
            q1 = q1.conjugated()
        elif costheta > 1:
            costheta = 1

        theta = math.acos(costheta)
        if abs(theta) < 0.01:
            Q.w = q2.w
            Q.x = q2.x
            Q.y = q2.y
            Q.z = q2.z
            return Q

        sintheta = math.sqrt(1.0 - costheta * costheta)
        if abs(sintheta) < 0.01:
            Q.w = (q1.w + q2.w) * 0.5
            Q.x = (q1.x + q2.x) * 0.5
            Q.y = (q1.y + q2.y) * 0.5
            Q.z = (q1.z + q2.z) * 0.5
            return Q

        ratio1 = math.sin((1 - t) * theta) / sintheta
        ratio2 = math.sin(t * theta) / sintheta

        Q.w = q1.w * ratio1 + q2.w * ratio2
        Q.x = q1.x * ratio1 + q2.x * ratio2
        Q.y = q1.y * ratio1 + q2.y * ratio2
        Q.z = q1.z * ratio1 + q2.z * ratio2
        return Q
    new_interpolate = classmethod(new_interpolate)

# Geometry
# Much maths thanks to Paul Bourke, http://astronomy.swin.edu.au/~pbourke
# ---------------------------------------------------------------------------

class Geometry:
    def _connect_unimplemented(self, other):
        raise AttributeError, 'Cannot connect %s to %s' % \
            (self.__class__, other.__class__)

    def _intersect_unimplemented(self, other):
        raise AttributeError, 'Cannot intersect %s and %s' % \
            (self.__class__, other.__class__)

    _intersect_point2 = _intersect_unimplemented
    _intersect_line2 = _intersect_unimplemented
    _intersect_circle = _intersect_unimplemented
    _connect_point2 = _connect_unimplemented
    _connect_line2 = _connect_unimplemented
    _connect_circle = _connect_unimplemented

    _intersect_point3 = _intersect_unimplemented
    _intersect_line3 = _intersect_unimplemented
    _intersect_sphere = _intersect_unimplemented
    _intersect_plane = _intersect_unimplemented
    _connect_point3 = _connect_unimplemented
    _connect_line3 = _connect_unimplemented
    _connect_sphere = _connect_unimplemented
    _connect_plane = _connect_unimplemented

    def intersect(self, other):
        raise NotImplementedError

    def connect(self, other):
        raise NotImplementedError

    def distance(self, other):
        c = self.connect(other)
        if c:
            return c.length
        return 0.0

def _intersect_point2_circle(P, C):
    return abs(P - C.c) <= C.r
    
def _intersect_line2_line2(A, B):
    d = B.v.y * A.v.x - B.v.x * A.v.y
    if d == 0:
        return None

    dy = A.p.y - B.p.y
    dx = A.p.x - B.p.x
    ua = (B.v.x * dy - B.v.y * dx) / d
    if not A._u_in(ua):
        return None
    ub = (A.v.x * dy - A.v.y * dx) / d
    if not B._u_in(ub):
        return None

    return Point2(A.p.x + ua * A.v.x,
                  A.p.y + ua * A.v.y)

def _intersect_line2_circle(L, C):
    a = L.v.magnitude_squared()
    b = 2 * (L.v.x * (L.p.x - C.c.x) + \
             L.v.y * (L.p.y - C.c.y))
    c = C.c.magnitude_squared() + \
        L.p.magnitude_squared() - \
        2 * C.c.dot(L.p) - \
        C.r ** 2
    det = b ** 2 - 4 * a * c
    if det < 0:
        return None
    sq = math.sqrt(det)
    u1 = (-b + sq) / (2 * a)
    u2 = (-b - sq) / (2 * a)
    if not L._u_in(u1):
        u1 = max(min(u1, 1.0), 0.0)
    if not L._u_in(u2):
        u2 = max(min(u2, 1.0), 0.0)

    # Tangent
    if u1 == u2:
        return Point2(L.p.x + u1 * L.v.x,
                      L.p.y + u1 * L.v.y)

    return LineSegment2(Point2(L.p.x + u1 * L.v.x,
                               L.p.y + u1 * L.v.y),
                        Point2(L.p.x + u2 * L.v.x,
                               L.p.y + u2 * L.v.y))

def _connect_point2_line2(P, L):
    d = L.v.magnitude_squared()
    assert d != 0
    u = ((P.x - L.p.x) * L.v.x + \
         (P.y - L.p.y) * L.v.y) / d
    if not L._u_in(u):
        u = max(min(u, 1.0), 0.0)
    return LineSegment2(P, 
                        Point2(L.p.x + u * L.v.x,
                               L.p.y + u * L.v.y))

def _connect_point2_circle(P, C):
    v = P - C.c
    v.normalize()
    v *= C.r
    return LineSegment2(P, Point2(C.c.x + v.x, C.c.y + v.y))

def _connect_line2_line2(A, B):
    d = B.v.y * A.v.x - B.v.x * A.v.y
    if d == 0:
        # Parallel, connect an endpoint with a line
        if isinstance(B, Ray2) or isinstance(B, LineSegment2):
            p1, p2 = _connect_point2_line2(B.p, A)
            return p2, p1
        # No endpoint (or endpoint is on A), possibly choose arbitrary point
        # on line.
        return _connect_point2_line2(A.p, B)

    dy = A.p.y - B.p.y
    dx = A.p.x - B.p.x
    ua = (B.v.x * dy - B.v.y * dx) / d
    if not A._u_in(ua):
        ua = max(min(ua, 1.0), 0.0)
    ub = (A.v.x * dy - A.v.y * dx) / d
    if not B._u_in(ub):
        ub = max(min(ub, 1.0), 0.0)

    return LineSegment2(Point2(A.p.x + ua * A.v.x, A.p.y + ua * A.v.y),
                        Point2(B.p.x + ub * B.v.x, B.p.y + ub * B.v.y))

def _connect_circle_line2(C, L):
    d = L.v.magnitude_squared()
    assert d != 0
    u = ((C.c.x - L.p.x) * L.v.x + (C.c.y - L.p.y) * L.v.y) / d
    if not L._u_in(u):
        u = max(min(u, 1.0), 0.0)
    point = Point2(L.p.x + u * L.v.x, L.p.y + u * L.v.y)
    v = (point - C.c)
    v.normalize()
    v *= C.r
    return LineSegment2(Point2(C.c.x + v.x, C.c.y + v.y), point)

def _connect_circle_circle(A, B):
    v = B.c - A.c
    v.normalize()
    return LineSegment2(Point2(A.c.x + v.x * A.r, A.c.y + v.y * A.r),
                        Point2(B.c.x - v.x * B.r, B.c.y - v.y * B.r))


class Point2(Vector2, Geometry):
    def __repr__(self):
        return 'Point2(%.2f, %.2f)' % (self.x, self.y)

    def intersect(self, other):
        return other._intersect_point2(self)

    def _intersect_circle(self, other):
        return _intersect_point2_circle(self, other)

    def connect(self, other):
        return other._connect_point2(self)

    def _connect_point2(self, other):
        return LineSegment2(other, self)
    
    def _connect_line2(self, other):
        c = _connect_point2_line2(self, other)
        if c:
            return c._swap()

    def _connect_circle(self, other):
        c = _connect_point2_circle(self, other)
        if c:
            return c._swap()

class Line2(Geometry):
    __slots__ = ['p', 'v']

    def __init__(self, *args):
        if len(args) == 3:
            assert isinstance(args[0], Point2) and \
                   isinstance(args[1], Vector2) and \
                   type(args[2]) == float
            self.p = args[0].copy()
            self.v = args[1] * args[2] / abs(args[1])
        elif len(args) == 2:
            if isinstance(args[0], Point2) and isinstance(args[1], Point2):
                self.p = args[0].copy()
                self.v = args[1] - args[0]
            elif isinstance(args[0], Point2) and isinstance(args[1], Vector2):
                self.p = args[0].copy()
                self.v = args[1].copy()
            else:
                raise AttributeError, '%r' % (args,)
        elif len(args) == 1:
            if isinstance(args[0], Line2):
                self.p = args[0].p.copy()
                self.v = args[0].v.copy()
            else:
                raise AttributeError, '%r' % (args,)
        else:
            raise AttributeError, '%r' % (args,)
        
        if not self.v:
            raise AttributeError, 'Line has zero-length vector'

    def __copy__(self):
        return self.__class__(self.p, self.v)

    copy = __copy__

    def __repr__(self):
        return 'Line2(<%.2f, %.2f> + u<%.2f, %.2f>)' % \
            (self.p.x, self.p.y, self.v.x, self.v.y)

    p1 = property(lambda self: self.p)
    p2 = property(lambda self: Point2(self.p.x + self.v.x, 
                                      self.p.y + self.v.y))

    def _apply_transform(self, t):
        self.p = t * self.p
        self.v = t * self.v

    def _u_in(self, u):
        return True

    def intersect(self, other):
        return other._intersect_line2(self)

    def _intersect_line2(self, other):
        return _intersect_line2_line2(self, other)

    def _intersect_circle(self, other):
        return _intersect_line2_circle(self, other)

    def connect(self, other):
        return other._connect_line2(self)

    def _connect_point2(self, other):
        return _connect_point2_line2(other, self)

    def _connect_line2(self, other):
        return _connect_line2_line2(other, self)

    def _connect_circle(self, other):
        return _connect_circle_line2(other, self)

class Ray2(Line2):
    def __repr__(self):
        return 'Ray2(<%.2f, %.2f> + u<%.2f, %.2f>)' % \
            (self.p.x, self.p.y, self.v.x, self.v.y)

    def _u_in(self, u):
        return u >= 0.0

class LineSegment2(Line2):
    def __repr__(self):
        return 'LineSegment2(<%.2f, %.2f> to <%.2f, %.2f>)' % \
            (self.p.x, self.p.y, self.p.x + self.v.x, self.p.y + self.v.y)

    def _u_in(self, u):
        return u >= 0.0 and u <= 1.0

    def __abs__(self):
        return abs(self.v)

    def magnitude_squared(self):
        return self.v.magnitude_squared()

    def _swap(self):
        # used by connect methods to switch order of points
        self.p = self.p2
        self.v *= -1
        return self

    length = property(lambda self: abs(self.v))

class Circle(Geometry):
    __slots__ = ['c', 'r']

    def __init__(self, center, radius):
        assert isinstance(center, Vector2) and type(radius) == float
        self.c = center.copy()
        self.r = radius

    def __copy__(self):
        return self.__class__(self.c, self.r)

    copy = __copy__

    def __repr__(self):
        return 'Circle(<%.2f, %.2f>, radius=%.2f)' % \
            (self.c.x, self.c.y, self.r)

    def _apply_transform(self, t):
        self.c = t * self.c

    def intersect(self, other):
        return other._intersect_circle(self)

    def _intersect_point2(self, other):
        return _intersect_point2_circle(other, self)

    def _intersect_line2(self, other):
        return _intersect_line2_circle(other, self)

    def connect(self, other):
        return other._connect_circle(self)

    def _connect_point2(self, other):
        return _connect_point2_circle(other, self)

    def _connect_line2(self, other):
        c = _connect_circle_line2(self, other)
        if c:
            return c._swap()

    def _connect_circle(self, other):
        return _connect_circle_circle(other, self)

# 3D Geometry
# -------------------------------------------------------------------------

def _connect_point3_line3(P, L):
    d = L.v.magnitude_squared()
    assert d != 0
    u = ((P.x - L.p.x) * L.v.x + \
         (P.y - L.p.y) * L.v.y + \
         (P.z - L.p.z) * L.v.z) / d
    if not L._u_in(u):
        u = max(min(u, 1.0), 0.0)
    return LineSegment3(P, Point3(L.p.x + u * L.v.x,
                                  L.p.y + u * L.v.y,
                                  L.p.z + u * L.v.z))

def _connect_point3_sphere(P, S):
    v = P - S.c
    v.normalize()
    v *= S.r
    return LineSegment3(P, Point3(S.c.x + v.x, S.c.y + v.y, S.c.z + v.z))

def _connect_point3_plane(p, plane):
    n = plane.n.normalized()
    d = p.dot(plane.n) - plane.k
    return LineSegment3(p, Point3(p.x - n.x * d, p.y - n.y * d, p.z - n.z * d))

def _connect_line3_line3(A, B):
    assert A.v and B.v
    p13 = A.p - B.p
    d1343 = p13.dot(B.v)
    d4321 = B.v.dot(A.v)
    d1321 = p13.dot(A.v)
    d4343 = B.v.magnitude_squared()
    denom = A.v.magnitude_squared() * d4343 - d4321 ** 2
    if denom == 0:
        # Parallel, connect an endpoint with a line
        if isinstance(B, Ray3) or isinstance(B, LineSegment3):
            return _connect_point3_line3(B.p, A)._swap()
        # No endpoint (or endpoint is on A), possibly choose arbitrary
        # point on line.
        return _connect_point3_line3(A.p, B)

    ua = (d1343 * d4321 - d1321 * d4343) / denom
    if not A._u_in(ua):
        ua = max(min(ua, 1.0), 0.0)
    ub = (d1343 + d4321 * ua) / d4343
    if not B._u_in(ub):
        ub = max(min(ub, 1.0), 0.0)
    return LineSegment3(Point3(A.p.x + ua * A.v.x,
                               A.p.y + ua * A.v.y,
                               A.p.z + ua * A.v.z),
                        Point3(B.p.x + ub * B.v.x,
                               B.p.y + ub * B.v.y,
                               B.p.z + ub * B.v.z))

def _connect_line3_plane(L, P):
    d = P.n.dot(L.v)
    if not d:
        # Parallel, choose an endpoint
        return _connect_point3_plane(L.p, P)
    u = (P.k - P.n.dot(L.p)) / d
    if not L._u_in(u):
        # intersects out of range, choose nearest endpoint
        u = max(min(u, 1.0), 0.0)
        return _connect_point3_plane(Point3(L.p.x + u * L.v.x,
                                            L.p.y + u * L.v.y,
                                            L.p.z + u * L.v.z), P)
    # Intersection
    return None

def _connect_sphere_line3(S, L):
    d = L.v.magnitude_squared()
    assert d != 0
    u = ((S.c.x - L.p.x) * L.v.x + \
         (S.c.y - L.p.y) * L.v.y + \
         (S.c.z - L.p.z) * L.v.z) / d
    if not L._u_in(u):
        u = max(min(u, 1.0), 0.0)
    point = Point3(L.p.x + u * L.v.x, L.p.y + u * L.v.y, L.p.z + u * L.v.z)
    v = (point - S.c)
    v.normalize()
    v *= S.r
    return LineSegment3(Point3(S.c.x + v.x, S.c.y + v.y, S.c.z + v.z), 
                        point)

def _connect_sphere_sphere(A, B):
    v = B.c - A.c
    v.normalize()
    return LineSegment3(Point3(A.c.x + v.x * A.r,
                               A.c.y + v.y * A.r,
                               A.c.x + v.z * A.r),
                        Point3(B.c.x + v.x * B.r,
                               B.c.y + v.y * B.r,
                               B.c.x + v.z * B.r))

def _connect_sphere_plane(S, P):
    c = _connect_point3_plane(S.c, P)
    if not c:
        return None
    p2 = c.p2
    v = p2 - S.c
    v.normalize()
    v *= S.r
    return LineSegment3(Point3(S.c.x + v.x, S.c.y + v.y, S.c.z + v.z), 
                        p2)

def _connect_plane_plane(A, B):
    if A.n.cross(B.n):
        # Planes intersect
        return None
    else:
        # Planes are parallel, connect to arbitrary point
        return _connect_point3_plane(A._get_point(), B)

def _intersect_point3_sphere(P, S):
    return abs(P - S.c) <= S.r
    
def _intersect_line3_sphere(L, S):
    a = L.v.magnitude_squared()
    b = 2 * (L.v.x * (L.p.x - S.c.x) + \
             L.v.y * (L.p.y - S.c.y) + \
             L.v.z * (L.p.z - S.c.z))
    c = S.c.magnitude_squared() + \
        L.p.magnitude_squared() - \
        2 * S.c.dot(L.p) - \
        S.r ** 2
    det = b ** 2 - 4 * a * c
    if det < 0:
        return None
    sq = math.sqrt(det)
    u1 = (-b + sq) / (2 * a)
    u2 = (-b - sq) / (2 * a)
    if not L._u_in(u1):
        u1 = max(min(u1, 1.0), 0.0)
    if not L._u_in(u2):
        u2 = max(min(u2, 1.0), 0.0)
    return LineSegment3(Point3(L.p.x + u1 * L.v.x,
                               L.p.y + u1 * L.v.y,
                               L.p.z + u1 * L.v.z),
                        Point3(L.p.x + u2 * L.v.x,
                               L.p.y + u2 * L.v.y,
                               L.p.z + u2 * L.v.z))

def _intersect_line3_plane(L, P):
    d = P.n.dot(L.v)
    if not d:
        # Parallel
        return None
    u = (P.k - P.n.dot(L.p)) / d
    if not L._u_in(u):
        return None
    return Point3(L.p.x + u * L.v.x,
                  L.p.y + u * L.v.y,
                  L.p.z + u * L.v.z)

def _intersect_plane_plane(A, B):
    n1_m = A.n.magnitude_squared()
    n2_m = B.n.magnitude_squared()
    n1d2 = A.n.dot(B.n)
    det = n1_m * n2_m - n1d2 ** 2
    if det == 0:
        # Parallel
        return None
    c1 = (A.k * n2_m - B.k * n1d2) / det
    c2 = (B.k * n1_m - A.k * n1d2) / det
    return Line3(Point3(c1 * A.n.x + c2 * B.n.x,
                        c1 * A.n.y + c2 * B.n.y,
                        c1 * A.n.z + c2 * B.n.z), 
                 A.n.cross(B.n))

class Point3(Vector3, Geometry):
    def __repr__(self):
        return 'Point3(%.2f, %.2f, %.2f)' % (self.x, self.y, self.z)

    def intersect(self, other):
        return other._intersect_point3(self)

    def _intersect_sphere(self, other):
        return _intersect_point3_sphere(self, other)

    def connect(self, other):
        return other._connect_point3(self)

    def _connect_point3(self, other):
        if self != other:
            return LineSegment3(other, self)
        return None

    def _connect_line3(self, other):
        c = _connect_point3_line3(self, other)
        if c:
            return c._swap()
        
    def _connect_sphere(self, other):
        c = _connect_point3_sphere(self, other)
        if c:
            return c._swap()

    def _connect_plane(self, other):
        c = _connect_point3_plane(self, other)
        if c:
            return c._swap()

class Line3:
    __slots__ = ['p', 'v']

    def __init__(self, *args):
        if len(args) == 3:
            assert isinstance(args[0], Point3) and \
                   isinstance(args[1], Vector3) and \
                   type(args[2]) == float
            self.p = args[0].copy()
            self.v = args[1] * args[2] / abs(args[1])
        elif len(args) == 2:
            if isinstance(args[0], Point3) and isinstance(args[1], Point3):
                self.p = args[0].copy()
                self.v = args[1] - args[0]
            elif isinstance(args[0], Point3) and isinstance(args[1], Vector3):
                self.p = args[0].copy()
                self.v = args[1].copy()
            else:
                raise AttributeError, '%r' % (args,)
        elif len(args) == 1:
            if isinstance(args[0], Line3):
                self.p = args[0].p.copy()
                self.v = args[0].v.copy()
            else:
                raise AttributeError, '%r' % (args,)
        else:
            raise AttributeError, '%r' % (args,)
        
        # XXX This is annoying.
        #if not self.v:
        #    raise AttributeError, 'Line has zero-length vector'

    def __copy__(self):
        return self.__class__(self.p, self.v)

    copy = __copy__

    def __repr__(self):
        return 'Line3(<%.2f, %.2f, %.2f> + u<%.2f, %.2f, %.2f>)' % \
            (self.p.x, self.p.y, self.p.z, self.v.x, self.v.y, self.v.z)

    p1 = property(lambda self: self.p)
    p2 = property(lambda self: Point3(self.p.x + self.v.x, 
                                      self.p.y + self.v.y,
                                      self.p.z + self.v.z))

    def _apply_transform(self, t):
        self.p = t * self.p
        self.v = t * self.v

    def _u_in(self, u):
        return True

    def intersect(self, other):
        return other._intersect_line3(self)

    def _intersect_sphere(self, other):
        return _intersect_line3_sphere(self, other)

    def _intersect_plane(self, other):
        return _intersect_line3_plane(self, other)

    def connect(self, other):
        return other._connect_line3(self)

    def _connect_point3(self, other):
        return _connect_point3_line3(other, self)

    def _connect_line3(self, other):
        return _connect_line3_line3(other, self)

    def _connect_sphere(self, other):
        return _connect_sphere_line3(other, self)

    def _connect_plane(self, other):
        c = _connect_line3_plane(self, other)
        if c:
            return c

class Ray3(Line3):
    def __repr__(self):
        return 'Ray3(<%.2f, %.2f, %.2f> + u<%.2f, %.2f, %.2f>)' % \
            (self.p.x, self.p.y, self.p.z, self.v.x, self.v.y, self.v.z)

    def _u_in(self, u):
        return u >= 0.0

class LineSegment3(Line3):
    def __repr__(self):
        return 'LineSegment3(<%.2f, %.2f, %.2f> to <%.2f, %.2f, %.2f>)' % \
            (self.p.x, self.p.y, self.p.z,
             self.p.x + self.v.x, self.p.y + self.v.y, self.p.z + self.v.z)

    def _u_in(self, u):
        return u >= 0.0 and u <= 1.0

    def __abs__(self):
        return abs(self.v)

    def magnitude_squared(self):
        return self.v.magnitude_squared()

    def _swap(self):
        # used by connect methods to switch order of points
        self.p = self.p2
        self.v *= -1
        return self

    length = property(lambda self: abs(self.v))

class Sphere:
    __slots__ = ['c', 'r']

    def __init__(self, center, radius):
        assert isinstance(center, Vector3) and type(radius) == float
        self.c = center.copy()
        self.r = radius

    def __copy__(self):
        return self.__class__(self.c, self.r)

    copy = __copy__

    def __repr__(self):
        return 'Sphere(<%.2f, %.2f, %.2f>, radius=%.2f)' % \
            (self.c.x, self.c.y, self.c.z, self.r)

    def _apply_transform(self, t):
        self.c = t * self.c

    def intersect(self, other):
        return other._intersect_sphere(self)

    def _intersect_point3(self, other):
        return _intersect_point3_sphere(other, self)

    def _intersect_line3(self, other):
        return _intersect_line3_sphere(other, self)

    def connect(self, other):
        return other._connect_sphere(self)

    def _connect_point3(self, other):
        return _connect_point3_sphere(other, self)

    def _connect_line3(self, other):
        c = _connect_sphere_line3(self, other)
        if c:
            return c._swap()

    def _connect_sphere(self, other):
        return _connect_sphere_sphere(other, self)

    def _connect_plane(self, other):
        c = _connect_sphere_plane(self, other)
        if c:
            return c

class Plane:
    # n.p = k, where n is normal, p is point on plane, k is constant scalar
    __slots__ = ['n', 'k']

    def __init__(self, *args):
        if len(args) == 3:
            assert isinstance(args[0], Point3) and \
                   isinstance(args[1], Point3) and \
                   isinstance(args[2], Point3)
            self.n = (args[1] - args[0]).cross(args[2] - args[0])
            self.n.normalize()
            self.k = self.n.dot(args[0])
        elif len(args) == 2:
            if isinstance(args[0], Point3) and isinstance(args[1], Vector3):
                self.n = args[1].normalized()
                self.k = self.n.dot(args[0])
            elif isinstance(args[0], Vector3) and type(args[1]) == float:
                self.n = args[0].normalized()
                self.k = args[1]
            else:
                raise AttributeError, '%r' % (args,)

        else:
            raise AttributeError, '%r' % (args,)
        
        if not self.n:
            raise AttributeError, 'Points on plane are colinear'

    def __copy__(self):
        return self.__class__(self.n, self.k)

    copy = __copy__

    def __repr__(self):
        return 'Plane(<%.2f, %.2f, %.2f>.p = %.2f)' % \
            (self.n.x, self.n.y, self.n.z, self.k)

    def _get_point(self):
        # Return an arbitrary point on the plane
        if self.n.z:
            return Point3(0., 0., self.k / self.n.z)
        elif self.n.y:
            return Point3(0., self.k / self.n.y, 0.)
        else:
            return Point3(self.k / self.n.x, 0., 0.)

    def _apply_transform(self, t):
        p = t * self._get_point()
        self.n = t * self.n
        self.k = self.n.dot(p)

    def intersect(self, other):
        return other._intersect_plane(self)

    def _intersect_line3(self, other):
        return _intersect_line3_plane(other, self)

    def _intersect_plane(self, other):
        return _intersect_plane_plane(self, other)

    def connect(self, other):
        return other._connect_plane(self)

    def _connect_point3(self, other):
        return _connect_point3_plane(other, self)

    def _connect_line3(self, other):
        return _connect_line3_plane(other, self)

    def _connect_sphere(self, other):
        return _connect_sphere_plane(other, self)

    def _connect_plane(self, other):
        return _connect_plane_plane(other, self)




#!/usr/bin/python
# -*- coding: utf-8 -*-
"""
perforated.py (c) 2010 rene.milk@uni-muenster.de,felix.albrecht@uni-muenster.de
Licence: WTFPLv2, see LICENSE.txt
"""

import sys, math, os, time
from optparse import OptionParser

# calculate points on the inner circle,
# beginning bottom left
# returns list of points in 2d
def generate_ellipse( center_x, center_y, radius_x, radius_y, id, n ):
	ellipse = []
	for i in range( 0, n ) : # means i = 0, ..., n-1:
		arc = math.pi * ( ( 5.0 / 4.0 ) + ( ( 2.0 * i ) / n ) )
		sinus_arc = math.sin( arc )
		cosinus_arc = math.cos( arc )
		x1 = center_x + ( radius_x * cosinus_arc )
		x2 = center_y + ( radius_y * sinus_arc )
		point = [ x1, x2 ]
		ellipse.append( [ point, id] )
	return ellipse

# calculate points on outer rectangle
# beginning bottom left
# rather trivial
# returns list of points in 2d
# [ [ point_x, point_y ], id_of_face_starting_clockwise_at_this_point ]
def generate_rectangle( length_x, length_y, ids ) :
	rectangle = []
	rectangle.append( [ [ 0.0, 0.0 ], ids[0] ] )
	rectangle.append( [ [ length_x, 0.0 ], ids[1] ] )
	rectangle.append( [ [ length_x, length_y ], ids[2] ] )
	rectangle.append( [ [ 0.0, length_y ], ids[3] ] )
	return rectangle

# write to trianlge
def write_to_triangle( ellipsoids, outer_rectangle, holes, triangle_filename ) :
	global number_of_points
	#if ( number_of_cells_x * number_of_cells_y * number_of_points * 3 ) > 19 :
		#print '\t[ writing                                 ]'
		#print '\t[',
		#five_percent_of_file_lines = int( number_of_cells_x * number_of_cells_y * number_of_points * 3 / 20.0 )
		#five_percents_of_file = 0
		#number_of_lines_written = 0
	file = open( triangle_filename, 'w' )
	# header
	file.write( '# ########################################################################\n' )
	file.write( '# %s\n' %( triangle_filename ) )
	file.write( '# written by generateHomogeneousPerforatedDomain.py on %s\n' %( time.strftime('%Y/%m/%d %H:%M:%S') ) )
	# get the points
	total_number_of_vertices = 0
	total_number_of_faces = 0
	for ellipsoid in ellipsoids :
		total_number_of_vertices += len( ellipsoid )
		total_number_of_faces += len( ellipsoid )
	total_number_of_vertices += len( outer_rectangle )
	total_number_of_faces += len( outer_rectangle )
	file.write( '# we have \t%i vertices\n' %( total_number_of_vertices ) )
	file.write( '#\t\t%i faces\n' %( total_number_of_faces ) )
	file.write( '#\t\t%i hole(s)\n' %( len( holes ) ) )
	file.write( '# ########################################################################\n' )
	file.write( '#\n' )
	# write the vertices
	file.write( '# these are the %i vertices\n' %( total_number_of_vertices ) )
	file.write( '%i 2 0 0\n' %( total_number_of_vertices ) )
	vertex_number = 0
	# of the inner ellipsoids
	for ellipsoid in ellipsoids :
		for point_with_id_on_ellipse in ellipsoid :
			point_on_ellipse = point_with_id_on_ellipse[ 0 ]
			x = point_on_ellipse[ 0 ]
			y = point_on_ellipse[ 1 ]
			file.write( '\t%i\t%f\t%f\n' %( vertex_number, x, y ) )
			vertex_number += 1
			#if ( number_of_cells_x * number_of_cells_y * number_of_points * 3 ) > 19 :
				#if ( number_of_lines_written % five_percent_of_file_lines ) == 0 :
					#if five_percents_of_file < 20 :
						#sys.stdout.write( '=' )
						#sys.stdout.flush()
						#five_percents_of_file += 1
	# and the outer rectangle
	for point_with_id_on_rectangle in outer_rectangle :
		point_on_rectangle = point_with_id_on_rectangle[ 0 ]
		x = point_on_rectangle[ 0 ]
		y = point_on_rectangle[ 1 ]
		file.write( '\t%i\t%f\t%f\n' %( vertex_number, x, y ) )
		#if ( number_of_cells_x * number_of_cells_y * number_of_points * 3 ) > 19 :
			#if ( number_of_lines_written % five_percent_of_file_lines ) == 0 :
				#if five_percents_of_file < 20 :
					#print '=',
					#sys.stdout.flush()
					#five_percents_of_file += 1
		vertex_number += 1
	file.write( '# these were the %i vertices\n' %( total_number_of_vertices ) )
	file.write( '#\n' )
	# write the faces with boundary id
	file.write( '# these are the %i faces\n' %( total_number_of_faces ) )
	file.write( '%i 1\n' %(total_number_of_faces) )
	face_number = 0
	# of the inner ellipsoids
	number_of_ellipsoids_written = 0
	for ellipsoid in ellipsoids :
		for point_with_id_on_ellipse in ellipsoid :
			id = point_with_id_on_ellipse[ 1 ]
			file.write( '\t%i\t%i\t%i\t%i\n' %( face_number, ( face_number % len( ellipsoid ) ) + number_of_ellipsoids_written, ( ( face_number +1 ) % len( ellipsoid ) ) + number_of_ellipsoids_written, id ) )
			face_number += 1
			#if ( number_of_cells_x * number_of_cells_y * number_of_points * 3 ) > 19 :
				#if ( number_of_lines_written % five_percent_of_file_lines ) == 0 :
					#if five_percents_of_file < 20 :
						#print '=',
						#sys.stdout.flush()
						#five_percents_of_file += 1
		number_of_ellipsoids_written += len( ellipsoid )
	# of the outer rectangle
	for point_with_id_on_rectangle in outer_rectangle :
		id = point_with_id_on_rectangle[ 1 ]
		file.write( '\t%i\t%i\t%i\t%i\n' %( face_number, face_number % len( outer_rectangle ) + ( total_number_of_vertices - len( outer_rectangle ) ) , ( face_number +1 ) % len( outer_rectangle ) + ( total_number_of_vertices - len( outer_rectangle ) ), id ) )
		face_number += 1
	file.write( '# these were the %i faces\n' %( total_number_of_faces ) )
	file.write( '#\n' )
	# write the holes
	file.write( '# this are the %i holes\n' %( len( holes ) ) )
	file.write( '%i\n' %( len( holes ) ) )
	for hole in holes :
		file.write('\t0\t%f\t%f\n' %( hole[ 0 ], hole[ 1 ] ) )
		#if ( number_of_cells_x * number_of_cells_y * number_of_points * 3 ) > 19 :
			#if ( number_of_lines_written % five_percent_of_file_lines ) == 0 :
				#if five_percents_of_file < 20 :
					#print '=',
					#sys.stdout.flush()
					#five_percents_of_file += 1
	file.write( '# these were the %i holes\n' %( len( holes ) ) )
	file.write( '#\n' )
	# ending
	file.write( '# ########################################################################\n' )
	file.write( '# end of file %s\n' %( triangle_filename ) )
	file.write( '# written by generateHomogeneousPerforatedDomain.py on %s\n' %( time.strftime('%Y/%m/%d %H:%M:%S') ) )
	file.write( '# ########################################################################\n' )
	#if ( number_of_cells_x * number_of_cells_y * number_of_points * 3 ) > 19 :
		#print ']'

def generate():
	parser = OptionParser()
	parser.add_option("-x", "--length_x", dest="domain_length_x", default=1.,
					help="domain_length_x", type='float')
	parser.add_option("-y", "--length_y", dest="domain_length_y", default=1.,
					help="domain_length_y", type='float')
	parser.add_option("-r", "--size_x", dest="standard_cell_size_x", default=0.1,
					help="standard_cell_size_x", type='float')
	parser.add_option("-s", "--size_y", dest="standard_cell_size_y", default=0.1,
					help="standard_cell_size_y", type='float')
	parser.add_option("-p", "--porosity", dest="porosity", default=0.4,
					help="porosity", type='float')
	parser.add_option("-n", "--num_points", dest="number_of_points_per_quarter", default=4,
					help="number of points per quarter", type='int')
	parser.add_option("-f", "--prefix", dest="filename_prefix", default='homogeneous_perforated_domain_2d_porosity',
					help="filename_prefix", type='string')
	(options, args) = parser.parse_args()

	# about the material
	# porosity = volume_of_fluid_part / overall_volume
	# sand: 0.36 ... 0.43
	# clay soil: 0.51 ... 0.58
	porosity = options.porosity
	#print 'porosity is %f' %( porosity )

	do_shift = False
	if do_shift :
		shift_x = -1.0
		shift_y = -1.0
	else :
		shift_x = 0.0
		shift_y = 0.0

	# about the domain
	domain_length_x = options.domain_length_x
	domain_length_y = options.domain_length_y

	# typical size of the standard cell
	standard_cell_size_x = options.standard_cell_size_x
	standard_cell_size_y = options.standard_cell_size_y
	standard_cell_area = standard_cell_size_x * standard_cell_size_y
	#print 'standard cell size is %f x %f, standard cell area is %f' %( standard_cell_size_x, standard_cell_size_y, standard_cell_area )

	# about the rectangles
	number_of_cells_x = int( domain_length_x / standard_cell_size_x )
	number_of_cells_y = int( domain_length_y / standard_cell_size_y )
	computed_length_domain_x = number_of_cells_x * standard_cell_size_x
	computed_length_domain_y = number_of_cells_y * standard_cell_size_y
	#print 'domain size is %f x %f' %( computed_length_domain_x, computed_length_domain_y )
	#print 'there are %i x %i = %i standard cells' %( number_of_cells_x, number_of_cells_y, number_of_cells_x * number_of_cells_y )

	# about the files to be written
	triangle_filename = '%s_%s_%i_holes.poly' %( options.filename_prefix, str( porosity ), number_of_cells_x * number_of_cells_y )

	# about the ellipses
	standard_circle_radius = math.sqrt( ( 1.0 - porosity ) * standard_cell_area * ( 1.0 / math.pi ) )
	ellipse_radius_x = standard_circle_radius
	ellipse_radius_y = standard_circle_radius
	ellipse_center_x = standard_cell_size_x / 2.0;
	ellipse_center_y = standard_cell_size_y / 2.0;
	#print 'standard circle radius is %f, standard circles center is ( %f, %f )' %( standard_circle_radius, ellipse_center_x, ellipse_center_y )

	# about the number of points to approximate the ellipses
	number_of_points_per_quarter = options.number_of_points_per_quarter

	# about the boundary ids
	id_of_ellipse_faces = 2
	id_of_bottom_rectangle_faces = 3
	id_of_right_rectangle_faces = 4
	id_of_top_rectangle_faces = 5
	id_of_left_rectangle_faces = 6

			
	print 'generating perforated domain with %i holes' %( number_of_cells_x * number_of_cells_y )
	#if ( number_of_cells_x * number_of_cells_y ) > 19 :
		#print '\t[ generating                              ]'
		#print '\t[',
		#five_percent_of_holes = int( number_of_cells_x * number_of_cells_y / 20.0 )
		#five_percents = 0
		#number_of_holes_written = 0

	# points per ellipse
	number_of_points = 4 * number_of_points_per_quarter

	# generate the ellipsoids
	ellipsoids = [ ]
	holes = [ ]
	for i in range( 0, number_of_cells_x ) :
		for j in range( 0, number_of_cells_y ) :
			center_x = ( i * standard_cell_size_x ) + ellipse_center_x
			center_y = ( j * standard_cell_size_y ) + ellipse_center_y
			ellipse = generate_ellipse( center_x, center_y, ellipse_radius_x, ellipse_radius_y, id_of_ellipse_faces, number_of_points )
			ellipsoids.append( ellipse )
			hole = [ center_x, center_y ]
			holes.append( hole )
			#if ( number_of_cells_x * number_of_cells_y ) > 19 :
				#if ( number_of_holes_written % five_percent_of_holes ) == 0 :
					#if five_percents < 20 :
						##print '=',
						#sys.stdout.write( '=' )
						#sys.stdout.flush()
						#five_percents += 1

	#if ( number_of_cells_x * number_of_cells_y ) > 19 :
		#print ']'
	print 'epsilon_suqare is %f' %( standard_cell_area )

	# generate the outer rectangle
	outer_rectangle = generate_rectangle( computed_length_domain_x, computed_length_domain_y, [ id_of_bottom_rectangle_faces, id_of_right_rectangle_faces, id_of_top_rectangle_faces, id_of_left_rectangle_faces ] )

	# shift if desired
	if do_shift :
		for ellipsoid in ellipsoids:
			for ellipsoid_point_with_id in ellipsoid :
				ellipsoid_point = ellipsoid_point_with_id[ 0 ]
				ellipsoid_point[ 0 ] += shift_x
				ellipsoid_point[ 1 ] += shift_y
		for outer_rectangle_point_with_id in outer_rectangle :
			outer_rectangle_point = outer_rectangle_point_with_id[ 0 ]
			outer_rectangle_point[ 0 ] += shift_x
			outer_rectangle_point[ 1 ] += shift_y
		for hole in holes :
			hole[ 0 ] += shift_x
			hole[ 1 ] += shift_y

	# write to triangle .poly file
	print 'writing to %s' %( triangle_filename )
	write_to_triangle( ellipsoids, outer_rectangle, holes, triangle_filename )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
gridhelper.py (c) 2009 rene.milk@uni-muenster.de

It is licensed to you under the terms of the WTFPLv2 (see below).

This program is free software. It comes with no warranty,
to the extent permitted by applicable law.


            DO WHAT THE FUCK YOU WANT TO PUBLIC LICENSE
                    Version 2, December 2004

 Copyright (C) 2004 Sam Hocevar
  14 rue de Plaisance, 75014 Paris, France
 Everyone is permitted to copy and distribute verbatim or modified
 copies of this license document, and changing it is allowed as long
 as the name is changed.

            DO WHAT THE FUCK YOU WANT TO PUBLIC LICENSE
   TERMS AND CONDITIONS FOR COPYING, DISTRIBUTION AND MODIFICATION

  0. You just DO WHAT THE FUCK YOU WANT TO.
"""

import sys
from PyQt4 import QtGui
from viewer import MeshViewer


def viewerTest(filename):
    app = QtGui.QApplication(['MeshViewer'])
    window = MeshViewer(filename)
    window.show()
    #app.exec_()

if __name__ == '__main__':
    do_diff = False
    if len(sys.argv) > 1:
        grids = sys.argv[1:]
    else:
        grids = ['test_grids/colored_aorta.ply','test_grids/tube.smesh',
             'test_grids/aorta_dune.smesh' ]
    for fn in grids:
        viewerTest(fn)


#!/usr/bin/python
# -*- coding: utf-8 -*-
"""
simple.py (c) 2010 rene.milk@uni-muenster.de
Licence: WTFPLv2, see LICENSE.txt
"""

from optparse import OptionParser

def generate():
	parser = OptionParser()
	parser.add_option("-a", "--length_a", dest="length_a", default=0.1,
					help="length_a", type='float')
	parser.add_option("-b", "--length_b", dest="length_b", default=0.7,
					help="length_b", type='float')
	parser.add_option("-c", "--length_c", dest="length_c", default=0.2,
					help="length_c", type='float')
	(options, args) = parser.parse_args()

	A = options.length_a
	B = options.length_b
	C = options.length_c

	nBids = 3

	verts = [ (0,0) , (0,A), (B-0.5*A,A), (B,A+C), (B+0.5*A,A+C) , (B,A) , (B+A,A) \
		, (B+A,0) , (B,0)  ]
	nVert = len(verts)

	bSegs = dict()
	bSegs[1] = [ (1,2), (2,3), (4,5), (5,6), (7,8), (8,0) ]
	bSegs[2] = [ (0,1) ]
	bSegs[3] = [ (3,4) ]
	bSegs[4] = [ (6,7) ]
	nBsegs = 9

	with open( "simple_a.poly", 'w' ) as out:

		out.write( '%d 2 0 %d\n'%(nVert,nBids) )
		cVert = 0
		for v in verts:
				out.write( '%d %f %f 0\n'%(cVert,v[0],v[1]) )
				cVert += 1
		out.write( '%d 1\n'%(nBsegs) )
		cBseg = 0
		for bid,vertIds in bSegs.items():
			for seg in vertIds:
				out.write( '%d %d %d %d\n'%(cBseg,seg[0],seg[1],bid)  )
				cBseg += 1

		out.write( '%d\n'%(0))



# -*- coding: utf-8 -*-
"""
Created on Mon Oct  1 16:14:20 2012

@author: r_milk01

supermodule.py (c) 2012 rene.milk@uni-muenster.de

It is licensed to you under the terms of the WTFPLv2.
"""

import git
import os
import random
import string
import tempfile
import control
import subprocess
import shutil
import copy

CFG_TPL = u'''#This file is intended for use on buildbot, do NOT set any compiler here
CONFIGURE_FLAGS="CXXFLAGS='-pedantic -DDEBUG -g3 -ggdb -O0 -std=c++0x -Wextra -Wall' \\
    --enable-fieldvector-size-is-method \\
    --disable-documentation "
'''

DOCS_TPL = u'''#This file is intended for use on buildbot
CONFIGURE_FLAGS="CXXFLAGS='-w -O0' \\
    --enable-fieldvector-size-is-method \\
    --enable-documentation "
'''

def generate(module_url, module_name, new_dir):
    temporary_name = ''.join(random.choice(string.ascii_letters + string.digits) for x in range(15))
    temporary_dir = os.path.join(tempfile.gettempdir(), temporary_name)

    url_tpl = 'http://users.dune-project.org/repositories/projects/%s.git'
    git.Repo.clone_from(module_url,
                        to_path=os.path.join(temporary_dir, module_name))
    common_path = os.path.join(temporary_dir, 'dune-common')
    git.Repo.clone_from(url_tpl % 'dune-common',
                        to_path=common_path)
    ctrl_path = os.path.join(temporary_dir, 'dune-common', 'bin', 'dunecontrol')
    ctrl = control.Dunecontrol(ctrl_path)

    git.Repo.init(new_dir)
    m_deps = ctrl.dependencies(module_name)
    deps = copy.deepcopy(m_deps)
    os.chdir(new_dir)

    def add_sub(dep, url=None):
        url = url or url_tpl % dep
        try:
            subprocess.check_output(['git', 'submodule', 'add', url, dep],
                                    stderr=subprocess.STDOUT)
        except subprocess.CalledProcessError, e:
            if e.output.find('already exists') < 0:
                raise e

    add_sub(module_name, module_url)
    for dep in m_deps['required'] + m_deps['suggested']:
        add_sub(dep)
    ctrl_path = os.path.join(new_dir, 'dune-common', 'bin', 'dunecontrol')
    ctrl = control.Dunecontrol(ctrl_path)
    def add_recursive(dep, cat):
        try:
            return ctrl.dependencies(dep)[cat]
        except control.ModuleMissing, m:
            add_sub(m.name)
            return add_recursive(dep,cat)
    for cat in ['required', 'suggested']:
        for dep in m_deps[cat]:
            deps[cat] = list(set(deps[cat] + add_recursive(dep,cat)))

    for dep in deps['suggested'] + deps['required']:
        add_sub(dep)

    for compiler in ['gcc-4.4', 'gcc-4.6', 'clang']:
        open(os.path.join(new_dir, 'config.opts.%s' % compiler),
             'wb').write(CFG_TPL)
    open(os.path.join(new_dir, 'config.opts.docs'), 'wb').write(DOCS_TPL)
    subprocess.check_output(['git', 'add', 'config*'],
                            stderr=subprocess.STDOUT)
    subprocess.check_output(['git', 'commit', '-m', 'intial commit'],
                            stderr=subprocess.STDOUT)
    shutil.rmtree(temporary_dir)

def get_dune_stuff():
    temporary_name = ''.join(random.choice(string.ascii_letters
                                + string.digits) for x in range(15))
    temporary_dir = os.path.join(tempfile.gettempdir(),
                                 temporary_name)
    url = 'http://users.dune-project.org/repositories/projects/dune-stuff.git'
    module = 'dune-stuff'
    generate(url, module, temporary_dir)
    return temporary_dir


#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
mesh.py (c) 2010 rene.milk@uni-muenster.de
Licence: WTFPLv2, see LICENSE.txt
"""

ply_header_tpl = """ply
format ascii 1.0
element vertex %d
property float x
property float y
property float z
property uchar red
property uchar green
property uchar blue
element face %d
property list uchar uint vertex_index
end_header
"""

from util.meshutil import MeshVertexList, Simplex3, BoundaryIdToColorMapper, \
	ColorToBoundaryIdMapper, vector, find_key
from gui.quadtree import Quadtree,Box
from util.euclid import Vector3
import random
from OpenGL.GL import *
from OpenGL.GLU import *
from gui.bounding import BoundingVolume
from collections import defaultdict
import smesh

try:
	#the spooled variant is new in 2.6
	from tempfile import SpooledTemporaryFile as TemporaryFile
except:
	from tempfile import TemporaryFile


def isAdjacentFace(fs,fa):
	for e in fs.edge_idx:
		e_ = ( e[1], e[0] )
		if e in fa.edge_idx or e_ in fa.edge_idx:
			return True
	return False

class Mesh():

	def __init__(self,dim):
		self.dim = dim
		self.draw_outline = True
		self.draw_faces = True
		self.outline_color = ( 1,1,1 )
		self.dl = None
		self.refine = False
		self.bid_color_mapping = defaultdict(list)
		self.vertex_list = MeshVertexList(self.dim)
		self.faces = []
		self.edges = []
		self.adj_points = dict()
		self.adj_faces = dict()
		
	def buildAdjacencyList(self):
		self.adj = dict()
		i_s = 0
		for face in self.faces:
			self.adj[i_s] = []
			for v in face.idx:
				self.adj[i_s] += self.adj_faces[v]
			#remove duplicate entries
			self.adj[i_s] = list(set(self.adj[i_s]))
			self.adj[i_s].remove(i_s)
			i_s += 1
			
# ---------------Parsing--------------------------------------------------------------------------------- #

	def parse(self, filename):
		self.faces = []
		self.edges = []
		self.adj_points = dict()
		self.adj_faces = dict()
		num_verts = 0
		num_faces = 0
		self.vertex_list = MeshVertexList(self.dim)
		if filename.endswith( '.ply' ):
			self.parsePLY( filename )
		else:
			smesh.read( self, filename )
		self.bid_color_mapping = defaultdict(list)
		for f in self.faces:
			self.bid_color_mapping[f.boundary_id].append(f.color)

	def parsePLY(self,fn):
		color_mapper = ColorToBoundaryIdMapper()
		fd = open( fn, 'r' )
		#a rather hard assumption...
		self.zero_based_idx = True
		#for line in fd.readlines():
		while fd:
			line = fd.readline()
			if line.startswith( 'end_header' ):
				break
			if line.startswith( 'element vertex' ):
				num_verts = int(line.split()[2])
				continue
			if line.startswith( 'element face' ):
				num_faces = int(line.split()[2])
				continue
		print 'read %d faces, %d vertices'%(num_faces,num_verts)
		for i in range(num_verts):
			line = fd.readline().split()
			try:
				v = vector( line[0], line[1], line[2] )
				if len(line) > 5:
					c = vector(line[3],line[4],line[5])
				else:
					c = Vector3()
				self.vertex_list.addVertex( v,c )
			except Exception, e:
				print e
				print line
				raise e
		for i in range(num_faces):
			line = map( lambda p: self.vertex_list.realIndex(int(p)), fd.readline().split()[1:] )
			v0 = line[0]
			v1 = line[1]
			v2 = line[2]
			s = Simplex3(v0,v1,v2,self.vertex_list,len(self.faces),None, color_mapper.getID( self.vertex_list.attribs[v0] ) )
			self.faces.append( s )
			for v in [v0,v1,v2]:
				if self.adj_points.has_key(v) :
					self.adj_points[v] += [v0,v1,v2]
				else:
					self.adj_points[v] = [v0,v1,v2]
				self.adj_points[v].remove(v)
				if self.adj_faces.has_key(v):
					self.adj_faces[v].append( len(self.faces) - 1 )
				else:
					self.adj_faces[v] = [ len(self.faces) - 1 ]

# --------Writing---------------------------------------------------------------------------------------- #

	def write(self,fn,bidMapper=ColorToBoundaryIdMapper()):
		if fn.endswith( '.ply' ):
			return self.writePLY( fn )
		else:
			return self.writeSMESH( fn, bidMapper )

	def writeSMESH(self,fn,bidMapper=ColorToBoundaryIdMapper()):
		out = None
		if not fn.endswith( '.smesh' ):
			fn += '.smesh'
		try:
			out = open(fn,'w')
		except:
			raise ImpossibleException()
		out.write( '#\n%d 3 0 %d\n'%(len(self.vertex_list),3) )#3 bids
		out.write( '# all vertices\n#\n' )
		cVert = int(not self.zero_based_idx)
		for i in range( len(self.vertex_list) ):
			v = self.vertex_list[i]
			out.write( '%d %f %f %f\n'%(cVert,v.x,v.y,v.z) )
			cVert += 1

		out.write( '\n# number of facets (= number of triangles), border marker\n#\n%d 1\n'%(len(self.faces)) )
		out.write( '# all faces\n#\n' )

		#we store a running avg normal per bid
		normals = dict()
		for f in self.faces:
			assert isinstance( f, Simplex3 )
			boundary_id = bidMapper.getID(f.color)
			if normals.has_key( boundary_id ):
				i,n = normals[boundary_id]
				normals[boundary_id] = (i+1, ( n * i + f.n )/(i+1) )
			else:
				normals[boundary_id] = (1, f.n )
			out.write( '%d %d %d %d %d\n'%(3,f.idx[0],f.idx[1],f.idx[2], boundary_id ) )
		out.write( '%d\n'%(0))
		out.close()
		normals_file = open( fn + '.normals' ,'w')
		for i,n in normals.iteritems():
			#for m in [n[1],n[1]/abs(n[1])]:
			m = n[1]
			normals_file.write( 'gd_%d: %f;%f;%f\n'%(i,m.x,m.y,m.z) )
		normals_file.close()
		return fn

	def writePLY(self,fn):
		out = None
		if not fn.endswith( '.ply' ):
			fn += '.ply'
		try:
			out = open(fn,'w')
		except:
			raise ImpossibleException()
		out.write( ply_header_tpl%(len(self.vertex_list),len(self.faces) ) )
		for i in range( len(self.vertex_list) ):
				v = self.vertex_list[i]
				c = self.vertex_list.attribs[i]
				#this will generate all black vertices, but conforms to format
				out.write( '%f %f %f %d %d %d\n'%(v.x,v.y,v.z,c.x,c.y,c.z ) )

		for f in self.faces:
			assert isinstance( f, Simplex3 )
			idx = map( lambda p: p + 1, f.idx )
			out.write( '%d %d %d %d\n'%(3,idx[0],idx[1],idx[2])  )

		out.flush()
		out.close()
		return fn

# ----------Drawing-------------------------------------------------------------------------------------- #

	def drawOutline(self,f,opacity=1):
		glLineWidth(2)
		glBegin(GL_LINE_LOOP)
		glNormal3fv(f.n())
		for v in f.v:
			glVertex(v())
		glEnd()
				
	def drawAdjacentFaces( self, face_idx ):
		for f_idx in self.adj[face_idx]:
			#self.drawOutline( self.faces[f_idx] )
			self.drawFace( self.faces[f_idx], 0.7 )
		#self.drawFace( self.faces[f_idx], 1.0 )
		self.drawOutline( self.faces[f_idx] )

	def draw(self, opacity=1.):
		glCallList(self.dl)

	def prepDraw(self,opacity=1.):
		for f in self.faces:
			f.reset(self.vertex_list)
		if self.dl :
			glDeleteLists( self.dl, 1 )
		self.dl = glGenLists(1)
		self.bounding_box = BoundingVolume( self )
		self.quad = Quadtree(self)
		glNewList(self.dl,GL_COMPILE)
		if self.draw_faces:
			glBegin(GL_TRIANGLES)					# Start Drawing The Pyramid
			for f in self.faces:
				self.drawFace(f)
			glEnd()

		if self.draw_outline:
			for f in self.faces:
				glColor4f(0,0,0,opacity)
				self.drawOutline(f)
		glEndList()

	def drawFaceIdx(s,idx):
		f = s.faces[idx]
		s.drawFace(f)

	def drawFace(s,f):
		n = f.n
		glNormal3fv(f.n())
		glColor3fv(f.color())
		for v in f.v:
			glVertex(v())

# ----------Mesh deformation-------------------------------------------------------------------------------------- #

	def laplacianDisplacement(self,N_1_p,p):
		n = len( N_1_p )
		s = Vector3()
		if n > 0:
			for j in N_1_p:
				s += self.vertex_list[j] - p
			s /= float(n)
		return s

	def scale(self,factor):
		for i in range(len(self.vertex_list)):
			self.vertex_list[i] *= factor
		self.prepDraw()

	def smooth(self,step):
		n = 0
		avg = 0.0
		for i in range( len(self.vertex_list) ):
			if self.adj_points.has_key(i):
				p_old = self.vertex_list[i]
				displacement = step * self.laplacianDisplacement( self.adj_points[i], self.vertex_list[i] )
				p_new = self.vertex_list[i] + displacement
				self.vertex_list[i] += displacement
				avg = ( abs(p_old)/abs(p_new) + n * avg ) / float( n + 1 )
				n += 1
		#self.scale( 0.91*avg )
		self.scale( 1.0*avg )
		self.prepDraw()

	def smooth2(self):
		n = 0
		avg = 0.0
		for f in self.faces:
			m = Vector3()
			area_sum = 0
			for f_n_id in self.adj[f.id]:
				f_n = self.faces[f_n_id]
				m += f_n.n * f_n.area
				area_sum += f_n.area
			m /= float(area_sum)
			self.faces[f_n_id].m = m / abs(m)
			displacement = [Vector3(),Vector3(),Vector3()]
			for i in range(3):
				p_old_i = f.v[i]
				area_n_sum = 0
				for t_id in self.adj_faces[f.idx[i]]:
					t = self.faces[t_id]
					area_n_sum += t.area
					v_t = (t.center - p_old_i).dot(t.m)*t.m
					displacement[i] += t.area * v_t
				displacement[i] /= area_n_sum
			for i in range(3):
				p_new = self.vertex_list[f.idx[i]] + displacement[i]
				self.vertex_list[f.idx[i]] = p_new
			#self.faces[f.id].reset(self.vertex_list)
		self.prepDraw()
		print 'smooth2 done'

	def noise(self,factor):
		for i in range( len(self.vertex_list) ):
			#self.vertex_list.verts[i] += random.gauss( factor, 1 ) * self.vertex_list.verts[i]
			self.vertex_list.verts[i] += random.random( ) * factor * self.vertex_list.verts[i]
		self.prepDraw()

	def equals(self,other):
		lists = ['faces', 'edges']
		for l in lists:
			diff = set(getattr(self,l)) ^ set(getattr(other,l))
			if len(diff) > 0:
				return False